Package: tidyBdE
Title: Retrieve Data from 'Banco de España'
Version: 0.6.1
Authors@R: 
    person("Diego", "H. Herrero", , "dev.dieghernan@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0001-8457-4658"))
Description: Tools for retrieving time series data from 'Banco de España'
    ('BdE') as 'tibble' objects. 'Banco de España' is the national central
    bank and, within the framework of the Single Supervisory Mechanism
    ('SSM'), the supervisor of the Spanish banking system alongside the
    European Central Bank. This package is not sponsored, endorsed or
    administered by 'Banco de España'.
License: GPL (>= 3)
URL: https://ropenspain.github.io/tidyBdE/,
        https://github.com/rOpenSpain/tidyBdE
BugReports: https://github.com/rOpenSpain/tidyBdE/issues
Depends: R (>= 4.1.0)
Imports: dplyr (>= 0.7.0), ggplot2 (>= 3.5.0), readr (>= 1.0.0), scales
        (>= 1.1.0), tibble (>= 3.0.0), tidyr, utils
Suggests: knitr, lifecycle, quarto, testthat (>= 3.0.0)
VignetteBuilder: quarto
Config/Needs/coverage: covr
Config/Needs/website: cpp11, devtools, progress, reactable, remotes,
        styler, tidyverse, ropenspain/rostemplate
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Config/testthat/parallel: true
Copyright: See file inst/COPYRIGHTS
Encoding: UTF-8
LazyData: true
X-schema.org-applicationCategory: Macroeconomics
X-schema.org-isPartOf: https://ropenspain.es/
X-schema.org-keywords: api, bde, cran, ggplot2, macroeconomics, r,
        r-package, ropenspain, rstats, series-data, spain
NeedsCompilation: no
Packaged: 2026-05-21 12:37:18 UTC; diego
Author: Diego H. Herrero [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-8457-4658>)
Maintainer: Diego H. Herrero <dev.dieghernan@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-21 12:50:02 UTC
Built: R 4.5.2; ; 2026-05-22 05:49:13 UTC; unix

Package: tidyBdE
Title: Retrieve Data from 'Banco de España'
Version: 0.6.1
Authors@R: 
    person("Diego", "H. Herrero", , "dev.dieghernan@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0001-8457-4658"))
Description: Tools for retrieving time series data from 'Banco de España'
    ('BdE') as 'tibble' objects. 'Banco de España' is the national central
    bank and, within the framework of the Single Supervisory Mechanism
    ('SSM'), the supervisor of the Spanish banking system alongside the
    European Central Bank. This package is not sponsored, endorsed or
    administered by 'Banco de España'.
License: GPL (>= 3)
URL: https://ropenspain.github.io/tidyBdE/,
        https://github.com/rOpenSpain/tidyBdE
BugReports: https://github.com/rOpenSpain/tidyBdE/issues
Depends: R (>= 4.1.0)
Imports: dplyr (>= 0.7.0), ggplot2 (>= 3.5.0), readr (>= 1.0.0), scales
        (>= 1.1.0), tibble (>= 3.0.0), tidyr, utils
Suggests: knitr, lifecycle, quarto, testthat (>= 3.0.0)
VignetteBuilder: quarto
Config/Needs/coverage: covr
Config/Needs/website: cpp11, devtools, progress, reactable, remotes,
        styler, tidyverse, ropenspain/rostemplate
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Config/testthat/parallel: true
Copyright: See file inst/COPYRIGHTS
Encoding: UTF-8
LazyData: true
X-schema.org-applicationCategory: Macroeconomics
X-schema.org-isPartOf: https://ropenspain.es/
X-schema.org-keywords: api, bde, cran, ggplot2, macroeconomics, r,
        r-package, ropenspain, rstats, series-data, spain
NeedsCompilation: no
Packaged: 2026-05-21 12:37:18 UTC; diego
Author: Diego H. Herrero [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-8457-4658>)
Maintainer: Diego H. Herrero <dev.dieghernan@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-21 12:50:02 UTC
Built: R 4.6.0; ; 2026-05-25 04:28:05 UTC; unix

# Generated by roxygen2: do not edit by hand

export(bde_catalog_load)
export(bde_catalog_search)
export(bde_catalog_update)
export(bde_check_access)
export(bde_ind_cpi_var)
export(bde_ind_euribor_12m_daily)
export(bde_ind_euribor_12m_monthly)
export(bde_ind_gdp_quarterly)
export(bde_ind_gdp_var)
export(bde_ind_ibex)
export(bde_ind_ibex_daily)
export(bde_ind_ibex_monthly)
export(bde_ind_population)
export(bde_ind_unemployment_rate)
export(bde_parse_dates)
export(bde_rose_pal)
export(bde_series_full_load)
export(bde_series_load)
export(bde_tidy_palettes)
export(bde_vivid_pal)
export(scale_color_bde_c)
export(scale_color_bde_d)
export(scale_colour_bde_c)
export(scale_colour_bde_d)
export(scale_fill_bde_c)
export(scale_fill_bde_d)
export(theme_tidybde)
importFrom(ggplot2,"%+replace%")
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,margin)
importFrom(ggplot2,rel)
importFrom(ggplot2,unit)
importFrom(utils,download.file)
importFrom(utils,read.csv2)

<!-- README.md is generated from README.qmd. Please edit that file -->

# tidyBdE <a href="https://ropenspain.github.io/tidyBdE/"><img src="man/figures/logo.png" alt="tidyBdE website" align="right" height="139"/></a>

<!-- badges: start -->

[![rOS-badge](https://ropenspain.github.io/rostemplate/reference/figures/ropenspain-badge.svg)](https://ropenspain.es/)
[![CRAN-status](https://www.r-pkg.org/badges/version/tidyBdE)](https://CRAN.R-project.org/package=tidyBdE)
[![CRAN-results](https://badges.cranchecks.info/worst/tidyBdE.svg)](https://cran.r-project.org/web/checks/check_results_tidyBdE.html)
[![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/tidyBdE?color=blue)](https://cran.r-project.org/package=tidyBdE)
[![On-CRAN](https://www.r-pkg.org/badges/ago/tidyBdE)](https://cran.r-project.org/web/checks/check_results_tidyBdE.html)
[![r-universe](https://ropenspain.r-universe.dev/badges/tidyBdE)](https://ropenspain.r-universe.dev/tidyBdE)
[![R-CMD-check](https://github.com/rOpenSpain/tidyBdE/actions/workflows/check-full.yaml/badge.svg)](https://github.com/rOpenSpain/tidyBdE/actions/workflows/check-full.yaml)
[![codecov](https://codecov.io/gh/ropenspain/tidyBdE/branch/main/graph/badge.svg)](https://app.codecov.io/gh/ropenspain/tidyBdE)
[![CodeFactor](https://www.codefactor.io/repository/github/ropenspain/tidybde/badge)](https://www.codefactor.io/repository/github/ropenspain/tidybde)
[![DOI](https://img.shields.io/badge/DOI-10.32614/CRAN.package.tidyBdE-blue)](https://doi.org/10.32614/CRAN.package.tidyBdE)
[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)

<!-- badges: end -->

**tidyBdE** is an **R** package that retrieves data from [Banco de
España](https://www.bde.es/webbe/en/estadisticas/recursos/descargas-completas.html).
Data are returned as a [tibble](https://tibble.tidyverse.org/), and the
package automatically detects the format of each time series (dates,
characters and numbers).

## Installation

Install **tidyBdE** from
[CRAN](https://CRAN.R-project.org/package=tidyBdE):

``` r
install.packages("tidyBdE")
```

You can install the development version of **tidyBdE** with:

``` r
pak::pak("ropenspain/tidyBdE")
```

Alternatively, you can install **tidyBdE** from the
[r-universe](https://ropenspain.r-universe.dev/tidyBdE):

``` r
# Install tidyBdE in R:
install.packages(
  "tidyBdE",
  repos = c(
    "https://ropenspain.r-universe.dev",
    "https://cloud.r-project.org"
  )
)
```

## Examples

Banco de España (**BdE**) provides several time series, either produced
by the institution itself or compiled from other sources, such as
[Eurostat](https://ec.europa.eu/eurostat) or [INE](https://www.ine.es/).

The basic entry point for searching series is the time series catalog.
You can search for series by name:

``` r
library(tidyBdE)

# Load packages for data handling and plotting.
library(ggplot2)
library(dplyr)
library(tidyr)

# Search for GBP in the "TC" (exchange rate) catalog.
xr_gbp <- bde_catalog_search("GBP", catalog = "TC")

xr_gbp |>
  select(Numero_secuencial, Descripcion_de_la_serie) |>
  # Display the table in the document.
  knitr::kable()
```

| Numero_secuencial | Descripcion_de_la_serie |
|---:|:---|
| 573214 | Tipo de cambio. Libras esterlinas por euro (GBP/EUR).Datos diarios |

<p class="caption">

Table 1: Search results
</p>

**Note:** BdE metadata is currently provided in Spanish only, so search
terms must be provided in Spanish to retrieve results. The institution
is working on an English version.

After finding a series, you can load the GBP/EUR exchange rate using the
sequential number reference (`Numero_secuencial`):

``` r
seq_number <- xr_gbp |>
  # Select the first record.
  slice(1) |>
  # Get the series ID.
  select(Numero_secuencial) |>
  # Convert to numeric.
  as.double()

# Extract the series.
time_series <- bde_series_load(seq_number, series_label = "EUR_GBP_XR") |>
  filter(Date >= "2010-01-01" & Date <= "2020-12-31") |>
  drop_na()

time_series
#> # A tibble: 2,816 × 2
#>    Date       EUR_GBP_XR
#>    <date>          <dbl>
#>  1 2010-01-04      0.891
#>  2 2010-01-05      0.900
#>  3 2010-01-06      0.899
#>  4 2010-01-07      0.900
#>  5 2010-01-08      0.893
#>  6 2010-01-11      0.899
#>  7 2010-01-12      0.897
#>  8 2010-01-13      0.895
#>  9 2010-01-14      0.890
#> 10 2010-01-15      0.881
#> # ℹ 2,806 more rows
```

### Plots

The package also provides a custom **ggplot2** theme based on BdE
publications:

``` r
ggplot(time_series, aes(x = Date, y = EUR_GBP_XR)) +
  geom_line(colour = bde_tidy_palettes(n = 1)) +
  geom_smooth(method = "gam", colour = bde_tidy_palettes(n = 2)[2]) +
  labs(
    title = "EUR/GBP Exchange Rate (2010-2020)",
    subtitle = "%",
    caption = "Source: BdE"
  ) +
  geom_vline(
    xintercept = as.Date("2016-06-23"),
    linetype = "dotted"
  ) +
  geom_label(aes(
    x = as.Date("2016-06-23"),
    y = 0.95,
    label = "Brexit"
  )) +
  coord_cartesian(ylim = c(0.7, 1)) +
  theme_tidybde()
```

<img src="man/figures/README-chart-1.png" style="width:100.0%"
alt="EUR/GBP Exchange Rate (2010-2020)" />

The package also provides convenience functions for selected
macroeconomic series, so you do not need to search for them manually:

``` r
# Data in long format.

plotseries <- bde_ind_gdp_var("GDP YoY", out_format = "long") |>
  bind_rows(
    bde_ind_unemployment_rate("Unemployment Rate", out_format = "long")
  ) |>
  drop_na() |>
  filter(Date >= "2010-01-01" & Date <= "2019-12-31")

ggplot(plotseries, aes(x = Date, y = serie_value)) +
  geom_line(aes(color = serie_name), linewidth = 1) +
  labs(
    title = "Spanish Economic Indicators (2010-2019)",
    subtitle = "%",
    caption = "Source: BdE"
  ) +
  theme_tidybde() +
  scale_color_bde_d(palette = "bde_vivid_pal") # Use a custom package palette.
```

<img src="man/figures/README-macroseries-1.png" style="width:100.0%"
alt="Spanish Economic Indicators (2010-2019)" />

### Palettes

Three custom palettes are available, based on those used by BdE in some
publications.

Apply these palettes to **ggplot2** plots using the scale functions
provided in the package (see
`help("scale_color_bde_d", package = "tidyBdE")`).

### A note on caching

You can use **tidyBdE** to create a local cache by setting the following
option:

``` r
options(bde_cache_dir = "./path/to/location")
```

When this option is set, **tidyBdE** looks for cached files in the
`bde_cache_dir` directory and loads them, speeding up data retrieval.

You can update the data after monthly or quarterly releases with the
following commands:

``` r
bde_catalog_update()

# Or use `update_cache = TRUE` in most functions.

bde_series_load("SOME ID", update_cache = TRUE)
```

## Disclaimer

This package is not sponsored, endorsed or administered by Banco de
España.

## Citation

<p>

H. Herrero D (2026). <em>tidyBdE: Retrieve Data from Banco de
España</em>.
<a href="https://doi.org/10.32614/CRAN.package.tidyBdE">doi:10.32614/CRAN.package.tidyBdE</a>.
<a href="https://ropenspain.github.io/tidyBdE/">https://ropenspain.github.io/tidyBdE/</a>.
</p>

A BibTeX entry for LaTeX users is:

    @Manual{R-tidyBdE,
      title = {{tidyBdE}: Retrieve Data from Banco de España},
      doi = {10.32614/CRAN.package.tidyBdE},
      author = {Diego {H. Herrero}},
      year = {2026},
      version = {0.6.1},
      url = {https://ropenspain.github.io/tidyBdE/},
      abstract = {Tools for retrieving time series data from Banco de España (BdE) as tibble objects. Banco de España is the national central bank and, within the framework of the Single Supervisory Mechanism (SSM), the supervisor of the Spanish banking system alongside the European Central Bank. This package is not sponsored, endorsed or administered by Banco de España.},
    }

#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
# import statements
#' @importFrom utils download.file read.csv2
#' @importFrom ggplot2 unit element_rect element_text element_blank element_line
NULL

#' BdE \CRANpkg{ggplot2} theme
#'
#' A custom \CRANpkg{ggplot2} theme based on the publications
#' of BdE.
#'
#' @family bde_plot
#'
#' @export
#'
#' @return A \CRANpkg{ggplot2}
#'   [`theme()`][ggplot2::theme_classic()].
#'
#' @inheritDotParams ggplot2::theme_classic
#'
#' @seealso [ggplot2::theme_classic()]
#'
#' @details
#'
#' Theme based on [ggplot2::theme_classic()].
#'
#' @importFrom ggplot2 %+replace% rel margin
#' @examplesIf bde_check_access()
#' \donttest{
#' library(ggplot2)
#' library(dplyr)
#' library(tidyr)
#'
#' series_TC <- bde_series_full_load("TC_1_1.csv")
#'
#' # If download was OK then plot
#' if (nrow(series_TC) > 0) {
#'   series_TC <- series_TC[c(1, 2)]
#'
#'   series_TC_pivot <- series_TC |>
#'     filter(
#'       Date >= "2020-01-01" & Date <= "2020-12-31",
#'       !is.na(series_TC[[2]])
#'     )
#'
#'   names(series_TC_pivot) <- c("x", "y")
#'
#'   ggplot(series_TC_pivot, aes(x = x, y = y)) +
#'     geom_line(linewidth = 0.8, color = bde_tidy_palettes(n = 1)) +
#'     labs(
#'       title = "Title",
#'       subtitle = "Some metric",
#'       caption = "Bank of Spain"
#'     ) +
#'     theme_tidybde()
#' }
#' }
#'
theme_tidybde <- function(...) {
  # nocov start
  ggplot2::theme_classic(...) %+replace%
    ggplot2::theme(
      line = element_line(linewidth = rel(0.5)),
      plot.background = element_rect(fill = "white", colour = NA),
      plot.margin = unit(rep(5.5, 4) * 3, "pt"),
      plot.title = element_text(
        margin = margin(b = 4),
        hjust = 0,
        vjust = 1,
        size = rel(1.1)
      ),
      plot.subtitle = element_text(
        hjust = 0,
        vjust = 1,
        size = rel(0.9),
        margin = margin(t = 4, b = 4)
      ),
      plot.caption = element_text(
        hjust = 1,
        vjust = 0,
        size = rel(0.75)
      ),
      panel.background = element_rect(fill = "white", colour = NA),
      panel.grid.major.y = element_line(
        colour = "grey70",
        linetype = "dashed"
      ),
      axis.title = element_blank(),
      axis.ticks.length = unit(-2.75, "pt"),
      axis.text.x.bottom = element_text(
        margin = margin(t = 7.5, b = 5, unit = "pt"),
        size = rel(0.9)
      ),
      axis.text.x.top = element_text(
        margin = margin(b = 7.5, t = 5, unit = "pt"),
        size = rel(0.9)
      ),
      axis.text.y.left = element_text(
        hjust = 1,
        margin = margin(r = 7.5, l = 5, unit = "pt"),
        size = rel(0.9)
      ),
      axis.text.y.right = element_text(
        hjust = 0,
        margin = margin(l = 7.5, r = 5, unit = "pt"),
        size = rel(0.9)
      ),
      legend.position = "bottom",
      legend.justification = c(0, 0),
      legend.title.align = 0,
      legend.title = element_blank(),
      legend.key = element_blank(),
      legend.key.width = unit(25, "pt"),
      legend.text = element_text(size = rel(0.9))
    )
  # nocov end
}

#' Check access to BdE
#'
#' @description
#' Check whether **R** can access resources at
#' <https://www.bde.es/webbe/en/estadisticas/recursos/descargas-completas.html>.
#'
#' @return A logical value.
#'
#' @keywords internal
#'
#' @examples
#' \donttest{
#' bde_check_access()
#' }
#' @export
bde_check_access <- function() {
  # Internal option, for checking purposes only
  # nocov start
  test <- getOption("bde_test_offline", NULL)
  if (isTRUE(test)) {
    message("dev> Testing offline")
    return(FALSE)
  }
  # nocov end

  url <- paste0(
    "https://www.bde.es/webbde/es/",
    "estadis/infoest/series/catalogo_tc.csv"
  )
  # nocov start
  access <-
    tryCatch(
      download.file(url, destfile = tempfile(), quiet = TRUE, mode = "wb"),
      warning = function(e) {
        FALSE
      }
    )

  if (isFALSE(access)) {
    res <- FALSE
  } else {
    res <- TRUE
  }
  res
  # nocov end
}

#' Skip tests
#' @noRd
skip_if_bde_offline <- function() {
  # nocov start
  if (bde_check_access()) {
    return(invisible(TRUE))
  }

  if (requireNamespace("testthat", quietly = TRUE)) {
    testthat::skip("tidyBdE> BdE API not reachable")
  }
  invisible()
  # nocov end
}

#' Load BdE catalogs
#'
#' @description
#' Load the time series catalogs provided by the BdE.
#'
#' @export
#'
#' @family catalog
#'
#' @encoding UTF-8
#'
#' @return A [tibble][tibble::tbl_df] object.
#'
#' @source
#'
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0("[time series bulk data download]",
#'       "(https://www.bde.es/webbe/en/estadisticas/recursos/",
#'       "descargas-completas.html)."))
#'
#' ```
#'
#' @param catalog A single value indicating the catalog to be updated or
#'   `"ALL"` as a shorthand. See **Details**.
#'
#' @param parse_dates Logical. If `TRUE`, dates are parsed using
#'   [bde_parse_dates()].
#'
#' @param update_cache Logical. If `TRUE`, the requested file is updated in
#'   `cache_dir`.
#'
#' @inheritParams bde_catalog_update
#'
#' @details
#' Accepted values for `catalog` are:
#'
#' ```{r, echo=FALSE}
#'
#' t <- tibble::tribble(
#' ~CODE, ~PUBLICATION, ~UPDATEFREQUENCY, ~FREQUENCY,
#' '`"BE"`', "Statistical Bulletin", "Daily", "Monthly",
#' '`"SI"`', "Summary Indicators", "Daily", "Daily",
#' '`"TC"`', "Exchange Rates", "Daily", "Daily",
#' '`"TI"`', "Interest Rates", "Daily", "Daily",
#' '`"PB"`', "Bank Lending Survey", "Quarterly", "Quarterly",
#' )
#'
#' names(t) <- paste0("**",
#'   c("CODE","PUBLICATION", "UPDATE FREQUENCY", "FREQUENCY"),
#'   "**")
#'
#' knitr::kable(t)
#' ```
#' Use `"ALL"` as a shorthand for updating all the catalogs at a glance.
#'
#' If the requested catalog is not cached, [bde_catalog_update()] is invoked.
#'
#' @examplesIf bde_check_access()
#' \donttest{
#' bde_catalog_load("TI", verbose = TRUE)
#' }
bde_catalog_load <- function(
  catalog = c("ALL", "BE", "SI", "TC", "TI", "PB"),
  parse_dates = TRUE,
  cache_dir = NULL,
  update_cache = FALSE,
  verbose = FALSE
) {
  catalog <- match.arg(catalog)
  # Validate
  valid_catalogs <- c("BE", "SI", "TC", "TI", "PB", "ALL")
  stopifnot(
    catalog %in% valid_catalogs,
    length(catalog) == 1,
    any(is.null(cache_dir), is.character(cache_dir)),
    is.logical(verbose),
    is.logical(parse_dates),
    is.logical(update_cache)
  )

  catalog_to_load <- catalog
  if (catalog == "ALL") {
    catalog_to_load <- setdiff(valid_catalogs, "ALL")
  }

  # Get cache dir
  cache_dir <- bde_hlp_cachedir(cache_dir = cache_dir, verbose = verbose)

  final_catalog <- lapply(catalog_to_load, function(x) {
    catalog_file <- paste0("catalogo_", tolower(x), ".csv")
    catalog_file <- file.path(cache_dir, catalog_file)
    has_cache <- file.exists(catalog_file)

    if (all(has_cache, isFALSE(update_cache))) {
      if (verbose) message("tidyBdE> Cached version of ", x, " detected")
    } else {
      # If no catalog is found or requested, update
      if (verbose) {
        message("tidyBdE> Need to download catalog ", x)
      }

      result <- bde_catalog_update(
        catalog = x,
        cache_dir = cache_dir,
        verbose = verbose
      )

      # On error return NULL
      if (any(is.data.frame(result), isFALSE(result))) {
        message("Download not available for ", x)
        return(NULL)
      }
    }

    # Catch error
    # nocov start
    r <- readLines(catalog_file)
    if (length(r) == 0) {
      message("File ", catalog_file, " not valid")
      return(invisible())
    }
    # nocov end

    catalog_load <- read.csv2(
      catalog_file,
      sep = ",",
      stringsAsFactors = FALSE,
      na.strings = "",
      header = FALSE,
      fileEncoding = "latin1"
    )

    # Convert names
    # Hard-coded, problematic on checks because UTF-8 values
    # Some OS would fail if this workaround is not used
    names_catalog <- c(
      "Nombre_de_la_serie",
      "Numero_secuencial",
      "Alias_de_la_serie",
      "Nombre_del_archivo_con_los_valores_de_la_serie",
      "Descripcion_de_la_serie",
      "Tipo_de_variable",
      "Codigo_de_unidades",
      "Exponente",
      "Numero_de_decimales",
      "Descripcion_de_unidades_y_exponente",
      "Frecuencia_de_la_serie",
      "Fecha_de_la_primera_observacion",
      "Fecha_de_la_ultima_observacion",
      "Numero_de_observaciones",
      "Titulo_de_la_serie",
      "Fuente",
      "Notas"
    )

    # Rename and delete first row
    names(catalog_load) <- names_catalog
    catalog_load <- catalog_load[-1, ]

    catalog_load <- bde_hlp_tochar(catalog_load)
    catalog_load
  })

  res_all <- unlist(lapply(final_catalog, is.null))

  if (any(res_all)) {
    msg <- paste0(catalog_to_load[res_all], collapse = ", ")
    message("tidyBdE> Could not load catalogs: ", msg)
  }

  # Guess formats

  # Unlist
  final_catalog <- dplyr::bind_rows(final_catalog)
  final_catalog <- bde_hlp_guess(
    final_catalog,
    preserve = names(final_catalog)[c(5, 15)]
  )

  # To tibble
  final_catalog <- tibble::as_tibble(final_catalog)

  # Parse dates
  if (parse_dates) {
    if (verbose) {
      message("tidyBdE> Parsing dates")
    }
    date_fields <- names(final_catalog)[grep(
      "Fecha",
      names(final_catalog),
      fixed = TRUE
    )]

    for (i in seq_along(date_fields)) {
      field <- date_fields[i]
      final_catalog[field] <- bde_parse_dates(final_catalog[[field]])
    }
  }
  final_catalog
}


#' Update BdE catalogs
#'
#' @description
#' Update the time series catalogs provided by BdE.
#'
#' @export
#'
#' @family catalog
#'
#' @encoding UTF-8
#'
#' @return None. Downloads the catalog file(s) to the local machine.
#'
#' @source
#'
#' ```{r, echo=FALSE, results='asis'}
#'
#' cat(paste0("[time series bulk data download]",
#'       "(https://www.bde.es/webbe/en/estadisticas/recursos/",
#'       "descargas-completas.html)."))
#'
#' ```
#'
#' @param catalog A character string indicating the catalog to be updated
#'   or `"ALL"` as a shorthand. See **Details**.
#' @param cache_dir A path to a cache directory. The directory can also be set
#'   via options with `options(bde_cache_dir = "path/to/dir")`.
#' @param verbose Logical `TRUE` or `FALSE`, display information useful for
#'   debugging.
#'
#' @details
#' Accepted values for `catalog` are:
#'
#' ```{r, echo=FALSE}
#'
#' t <- tibble::tribble(
#' ~CODE, ~PUBLICATION, ~UPDATEFREQUENCY, ~FREQUENCY,
#' '`"BE"`', "Statistical Bulletin", "Daily", "Monthly",
#' '`"SI"`', "Summary Indicators", "Daily", "Daily",
#' '`"TC"`', "Exchange Rates", "Daily", "Daily",
#' '`"TI"`', "Interest Rates", "Daily", "Daily",
#' '`"PB"`', "Bank Lending Survey", "Quarterly", "Quarterly",
#' )
#'
#' names(t) <- paste0("**",
#'   c("CODE","PUBLICATION", "UPDATE FREQUENCY", "FREQUENCY"),
#'   "**")
#'
#' knitr::kable(t)
#'
#' ```
#' Use `"ALL"` as a shorthand for updating all the catalogs at a glance.
#'
#' @examplesIf bde_check_access()
#' \donttest{
#' bde_catalog_update("TI", verbose = TRUE)
#' }
bde_catalog_update <- function(
  catalog = c("ALL", "BE", "SI", "TC", "TI", "PB"),
  cache_dir = NULL,
  verbose = FALSE
) {
  catalog <- match.arg(catalog)
  # Validate
  valid_catalogs <- c("BE", "SI", "TC", "TI", "PB", "ALL")
  stopifnot(
    catalog %in% valid_catalogs,
    any(is.null(cache_dir), is.character(cache_dir)),
    is.logical(verbose)
  )

  # nocov start
  if (!bde_check_access()) {
    tbl <- bde_hlp_return_null()
    return(tbl)
  }
  # nocov end

  # Get cache dir
  cache_dir <- bde_hlp_cachedir(cache_dir = cache_dir, verbose = verbose)

  # Loop and download
  catalog_download <- catalog
  if ("ALL" %in% catalog) {
    catalog_download <- valid_catalogs[valid_catalogs != "ALL"]
  }

  if (verbose) {
    message(
      "tidyBdE> Updating catalogs: ",
      paste0(catalog_download, collapse = ", ")
    )
  }

  res <- lapply(catalog_download, function(x) {
    serie <- x
    base_url <- paste0(
      "https://www.bde.es/webbe/es/estadisticas/",
      "compartido/datos/csv/"
    )
    catalog_file <- paste0("catalogo_", tolower(serie), ".csv")

    full_url <- paste0(base_url, catalog_file)
    local_file <- file.path(cache_dir, catalog_file)

    # Download
    result <- bde_hlp_download(
      url = full_url,
      local_file = local_file,
      verbose = verbose
    )
    result
  })

  invisible(res)
}

#' Search BdE catalogs
#'
#' @description
#' Search for keywords on the time series catalogs.
#'
#' @export
#'
#' @family catalog
#'
#' @return A [tibble][tibble::tbl_df] object with the results of the query.
#'
#' @param pattern [`regex`][base::regex] pattern to search. See **Details**
#'   and **Examples**.
#'
#' @inheritDotParams bde_catalog_load
#'
#' @encoding UTF-8
#'
#' @details
#' **Note:** BdE files are currently provided only in Spanish. Therefore, search
#' terms should be provided in Spanish to obtain search results.
#'
#' This function uses [base::grep()] function for finding matches on the
#' catalogs. You can pass [regular expressions][base::regex] to broaden the
#' search.
#'
#' @seealso [bde_catalog_load()], [base::regex]
#'
#' @examplesIf bde_check_access()
#' \donttest{
#' # Simple search (needs to be in Spanish)
#' # !! PIB [es] == GDP [en]
#'
#' bde_catalog_search("PIB")
#'
#' # More complex - Single
#' bde_catalog_search("Francia(.*)PIB")
#'
#' # Even more complex - Double
#' bde_catalog_search("Francia(.*)PIB|Italia(.*)PIB|Alemania(.*)PIB")
#' }
bde_catalog_search <- function(pattern, ...) {
  # Extract info
  catalog_search <- bde_catalog_load(...)

  # nocov start
  if (nrow(catalog_search) == 0) {
    tbl <- bde_hlp_return_null()
    return(tbl)
  }

  if (!tibble::is_tibble(catalog_search)) {
    message("Catalogs corrupted. Try redownloading with bde_catalog_update()")
    return(invisible())
  }
  # nocov end

  # Index lookup columns
  col_ind <- c(2, 3, 4, 5, 15)

  search_match_rows <- NULL
  # Loop through cols
  for (i in col_ind) {
    search_match_rows <- unique(
      c(
        search_match_rows,
        grep(pattern, catalog_search[[i]], ignore.case = TRUE, useBytes = TRUE)
      )
    )
  }

  search_results <- catalog_search[search_match_rows, ]
  if (nrow(search_results) == 0) {
    stop("tidyBdE> No matches found for ", pattern)
  }
  search_results
}

test_that("Catalogs offline", {
  skip_on_cran()
  skip_if_bde_offline()

  dir <- file.path(tempdir(), "test_catalogs")

  expect_silent(bde_catalog_update("TI", cache_dir = dir))

  options(bde_test_offline = TRUE)

  expect_message(bde_catalog_update("TC", cache_dir = dir))

  table1 <- bde_catalog_load("TI", cache_dir = dir)
  all <- bde_catalog_load("ALL", cache_dir = dir)

  expect_equal(nrow(table1), nrow(all))

  null <- bde_catalog_load("TC", cache_dir = dir)

  expect_s3_class(null, "data.frame")
  expect_equal(nrow(null), 0)

  search <- as.double(table1[1, 2])

  s <- bde_catalog_search(search, catalog = "TI", cache_dir = dir)

  expect_gte(nrow(s), 1)

  s2 <- bde_catalog_search(search, catalog = "TC", cache_dir = dir)
  expect_equal(nrow(s2), 0)

  # Now try online
  options(bde_test_offline = FALSE)

  nonull <- bde_catalog_load("TC", cache_dir = dir)
  expect_gt(nrow(nonull), 0)
})

test_that("Messages", {
  skip_on_cran()
  skip_if_bde_offline()

  dir <- file.path(tempdir(), "test_catalogs2")

  options(bde_test_offline = FALSE)
  expect_message(bde_catalog_load("TC", verbose = TRUE, cache_dir = dir))
  expect_message(bde_catalog_load("TC", verbose = TRUE, cache_dir = dir))
})

test_that("Old tests: Catalogs", {
  skip_on_cran()
  skip_if_bde_offline()

  # Test load catalogs----
  expect_error(bde_catalog_load("aa"))
  expect_error(bde_catalog_search("xxxxxxx", catalog = "IE"))

  expect_message(bde_catalog_load("TC", cache_dir = tempdir(), verbose = TRUE))
  expect_message(bde_catalog_load("TC", cache_dir = NULL, verbose = TRUE))
  expect_message(bde_catalog_load(
    "TC",
    cache_dir = file.path(tempdir(), "aa"),
    verbose = TRUE
  ))

  expect_silent(bde_catalog_load("ALL"))

  # Test update catalogs----

  expect_error(bde_catalog_update("aa"))

  expect_message(bde_catalog_update(
    "TC",
    cache_dir = tempdir(),
    verbose = TRUE
  ))
  expect_silent(bde_catalog_update("ALL", cache_dir = tempdir()))
  expect_silent(bde_catalog_update("TC", cache_dir = tempdir()))

  # Testing options cache dir
  init_cache_dir <- getOption("bde_cache_dir")
  options(bde_cache_dir = file.path(tempdir(), "test"))
  expect_message(bde_catalog_update("TC", verbose = TRUE))
  # Reset
  options(bde_cache_dir = init_cache_dir)

  # Test bde_catalog_search----

  expect_error(bde_catalog_search())

  expect_silent(bde_catalog_search("Euro", catalog = "TC"))
})

test_that("Fully Deprecation of Series", {
  expect_error(bde_catalog_update("IE", cache_dir = tempdir()))
  expect_error(bde_catalog_update("CF", cache_dir = tempdir()))
})

#' BdE scales for \CRANpkg{ggplot2}
#'
#' @description
#'
#' Scales to be used with the \CRANpkg{ggplot2} package. Discrete palettes are
#' named `scale_*_bde_d`, while continuous palettes are named `scale_*_bde_c`.
#'
#' @seealso [ggplot2::discrete_scale()], [ggplot2::continuous_scale()]
#'
#' @family bde_plot
#'
#' @export
#'
#' @return A \CRANpkg{ggplot2} color scale.
#'
#' @rdname scales_bde
#'
#' @name scales_bde
#'
#' @param palette Name of the BdE palette to apply. See [bde_tidy_palettes()]
#'   for details.
#'
#' @inheritParams bde_tidy_palettes
#' @inheritParams ggplot2::continuous_scale
#'
#' @param ... Further arguments of [ggplot2::discrete_scale()] or
#'   [ggplot2::continuous_scale()].
#'
#' @examples
#' library(ggplot2)
#'
#' set.seed(596)
#' txsamp <- subset(
#'   txhousing,
#'   city %in% c(
#'     "Houston", "Fort Worth",
#'     "San Antonio", "Dallas", "Austin"
#'   )
#' )
#'
#' ggplot(txsamp, aes(x = sales, y = median)) +
#'   geom_point(aes(colour = city)) +
#'   scale_color_bde_d() +
#'   theme_minimal()
#'
#' ggplot(txsamp, aes(x = sales, y = median)) +
#'   geom_point(aes(colour = city)) +
#'   scale_color_bde_d("bde_qual_pal") +
#'   theme_minimal()
#'
scale_color_bde_d <- function(
  palette = c("bde_vivid_pal", "bde_rose_pal", "bde_qual_pal"),
  alpha = NULL,
  rev = FALSE,
  ...
) {
  palette <- match.arg(palette)

  cols_v <- bde_tidy_palettes(palette = palette, alpha = alpha, rev = rev)
  pal <- scales::manual_pal(cols_v)

  ggplot2::discrete_scale(aesthetics = "color", palette = pal, ...)
}

#' @rdname scales_bde
#' @name scales_bde
#' @export
#' @usage NULL
scale_colour_bde_d <- scale_color_bde_d

#' @rdname scales_bde
#' @name scales_bde
#' @export
scale_fill_bde_d <- function(
  palette = c("bde_vivid_pal", "bde_rose_pal", "bde_qual_pal"),
  alpha = NULL,
  rev = FALSE,
  ...
) {
  palette <- match.arg(palette)

  cols_v <- bde_tidy_palettes(palette = palette, alpha = alpha, rev = rev)
  pal <- scales::manual_pal(cols_v)

  ggplot2::discrete_scale(aesthetics = "fill", palette = pal, ...)
}


#' @rdname scales_bde
#' @name scales_bde
#' @export
scale_color_bde_c <- function(
  palette = c("bde_rose_pal", "bde_vivid_pal", "bde_qual_pal"),
  alpha = NULL,
  rev = FALSE,
  guide = "colorbar",
  ...
) {
  palette <- match.arg(palette)

  cols <- switch(palette,
    "bde_vivid_pal" = bde_tidy_palettes(
      6,
      "bde_vivid_pal",
      alpha = alpha,
      rev = rev
    ),
    "bde_qual_pal" = bde_tidy_palettes(
      6,
      "bde_qual_pal",
      alpha = alpha,
      rev = rev
    ),
    "bde_rose_pal" = bde_tidy_palettes(
      6,
      "bde_rose_pal",
      alpha = alpha,
      rev = rev
    )[c(1, 2, 3, 6, 5, 4)]
  )
  ggplot2::continuous_scale(
    aesthetics = "color",
    palette = scales::gradient_n_pal(cols),
    guide = guide,
    ...
  )
}

#' @rdname scales_bde
#' @name scales_bde
#' @export
#' @usage NULL
scale_colour_bde_c <- scale_color_bde_c

#' @rdname scales_bde
#' @name scales_bde
#' @export
scale_fill_bde_c <- function(
  palette = c("bde_rose_pal", "bde_vivid_pal", "bde_qual_pal"),
  alpha = NULL,
  rev = FALSE,
  guide = "colorbar",
  ...
) {
  palette <- match.arg(palette)

  cols <- switch(palette,
    "bde_vivid_pal" = bde_tidy_palettes(
      6,
      "bde_vivid_pal",
      alpha = alpha,
      rev = rev
    ),
    "bde_qual_pal" = bde_tidy_palettes(
      6,
      "bde_qual_pal",
      alpha = alpha,
      rev = rev
    ),
    "bde_rose_pal" = bde_tidy_palettes(
      6,
      "bde_rose_pal",
      alpha = alpha,
      rev = rev
    )[c(1, 2, 3, 6, 5, 4)]
  )

  ggplot2::continuous_scale(
    aesthetics = "fill",
    palette = scales::gradient_n_pal(cols),
    guide = guide,
    ...
  )
}

test_that("Discrete scale", {
  d <- data.frame(x = 1:5, y = 1:5, z = 21:25, l = letters[1:5])

  p <- ggplot2::ggplot(d) +
    ggplot2::geom_point(ggplot2::aes(x, y, color = l))

  init <- ggplot2::layer_data(p)$colour
  p2 <- p + scale_color_bde_d()

  mod <- ggplot2::layer_data(p2)$colour
  expect_true(!any(init %in% mod))

  # Renamed
  p3 <- p + scale_colour_bde_d()
  mod3 <- ggplot2::layer_data(p3)$colour
  expect_identical(mod, mod3)

  # Alpha

  p3 <- p + scale_colour_bde_d(alpha = 0.9)

  mod_alpha <- ggplot2::layer_data(p3)$colour

  expect_true(all(ggplot2::alpha(mod, 0.9) == mod_alpha))

  # Another pal
  p4 <- p + scale_color_bde_d(palette = "bde_rose_pal")
  mod4 <- ggplot2::layer_data(p4)$colour

  expect_false(any(mod == mod4))

  # Another pal
  p5 <- p + scale_color_bde_d(palette = "bde_qual_pal")
  mod5 <- ggplot2::layer_data(p5)$colour

  expect_false(any(mod == mod5))

  # Another aes
  pf <- ggplot2::ggplot(d) +
    ggplot2::geom_point(ggplot2::aes(x, y, fill = l), shape = 21)

  pfill <- pf + scale_fill_bde_d()
  colfill <- ggplot2::layer_data(pfill)$fill

  expect_identical(mod, colfill)

  pfill2 <- pf + scale_fill_bde_d(palette = "bde_rose_pal")
  colfill2 <- ggplot2::layer_data(pfill2)$fill

  expect_identical(mod4, colfill2)
})

test_that("Continous scale", {
  d <- data.frame(x = 1:5, y = 1:5, z = 21:25, l = letters[1:5])

  p <- ggplot2::ggplot(d) +
    ggplot2::geom_point(ggplot2::aes(x, y, colour = z))

  init <- ggplot2::layer_data(p)$colour
  p2 <- p + scale_color_bde_c()

  mod <- ggplot2::layer_data(p2)$colour
  expect_true(!any(init %in% mod))

  # Renamed
  p3 <- p + scale_colour_bde_c()
  mod3 <- ggplot2::layer_data(p3)$colour
  expect_identical(mod, mod3)

  # Alpha

  p3 <- p + scale_colour_bde_c(alpha = 0.9)

  mod_alpha <- ggplot2::layer_data(p3)$colour

  expect_true(all(ggplot2::alpha(mod, 0.9) == mod_alpha))

  # Another pal
  p4 <- p + scale_color_bde_c(palette = "bde_vivid_pal")
  mod4 <- ggplot2::layer_data(p4)$colour

  expect_false(any(mod == mod4))

  # Another aes
  pf <- ggplot2::ggplot(d) +
    ggplot2::geom_point(ggplot2::aes(x, y, fill = z), shape = 21)

  pfill <- pf + scale_fill_bde_c()
  colfill <- ggplot2::layer_data(pfill)$fill

  expect_identical(mod, colfill)

  pfill2 <- pf + scale_fill_bde_c(palette = "bde_vivid_pal")
  colfill2 <- ggplot2::layer_data(pfill2)$fill

  expect_identical(mod4, colfill2)
})

#' Parse dates
#'
#' Tries to parse strings representing dates using [as.Date()]
#'
#' @export
#'
#' @family utils
#'
#' @return A [`Date`][as.Date()] object.
#'
#' @seealso [as.Date()]
#'
#' @param dates_to_parse Dates to parse
#'
#' @description
#' This function is tailored for date formats used in this package and may fail
#' with other datasets. See **Examples** for formats that are supported.
#'
#' ## Date Formats
#'
#' ```{r, echo=FALSE}
#'
#' dates <- tibble::tribble(
#'   ~FREQUENCY, ~FORMAT, ~EXAMPLES,
#'   "**Daily / Business day**", "DD MMMMYYYY", "*02 FEB2019*",
#'   "**Monthly**", "MMM YYYY", "*MAR 2020*",
#'   "**Quarterly**", paste(
#'     "MMM YYYY, where MMM is the first",
#'     "or the last month of the",
#'     "quarter, depending on the value of",
#'     "its variable OBSERVED."
#'   ),
#'   "For the first quarter of 2020: *ENE 2020, MAR 2020*",
#'   "**Half-yearly**", paste(
#'     "MMM YYYY, where MMM is the first or the last month",
#'     "of the halfyear period, depending on the value of its",
#'     "variable OBSERVED."
#'   ),
#'   "For the first half of 2020: *ENE 2020, JUN 2020*",
#'   "**Annual**", "YYYY", "*2020*"
#' )
#' names(dates) <- paste0("**", names(dates), "**")
#'
#' knitr::kable(dates)
#' ```
#'
#' @examples
#' # Formats parsed
#' would_parse <- c(
#'   "02 FEB2019", "15 ABR 1890", "MAR 2020", "ENE2020",
#'   "2020", "12-1993", "01-02-2014", "01/02/1990"
#' )
#'
#' parsed_ok <- bde_parse_dates(would_parse)
#'
#' class(parsed_ok)
#'
#' tibble::tibble(raw = would_parse, parsed = parsed_ok)
#'
#' #-----------------------------------
#'
#' # Formats not admitted
#' wont_parse <- c("JAN2001", "2010-01-12", "01 APR 2017", "01/31/1990")
#'
#' parsed_fail <- bde_parse_dates(wont_parse)
#'
#' class(parsed_fail)
#'
#' tibble::tibble(raw = wont_parse, parsed = parsed_fail)
bde_parse_dates <- function(dates_to_parse) {
  dateformat <- gsub(" ", "", toupper(dates_to_parse), fixed = TRUE)
  dateformat <- gsub("-", "", dateformat, fixed = TRUE)
  dateformat <- gsub("/", "", dateformat, fixed = TRUE)

  months_esp <- c(
    "ENE",
    "FEB",
    "MAR",
    "ABR",
    "MAY",
    "JUN",
    "JUL",
    "AGO",
    "SEP",
    "OCT",
    "NOV",
    "DIC"
  )

  # Format months
  for (i in seq_along(months_esp)) {
    dateformat <- gsub(months_esp[i], sprintf("%02d", i), dateformat)
  }

  # Final format: dd-mm-yyyy
  for (j in seq_along(dateformat)) {
    s2 <- dateformat[j]

    if (is.na(s2) || nchar(s2) < 4) {
      # Return NULL
      dateformat[j] <- NA
    } else if (nchar(s2) == 4) {
      # This is just year, add day, month
      dateformat[j] <- paste0("3112", s2)
    } else if (nchar(s2) == 6) {
      # Month Year, add day
      dateformat[j] <- paste0("01", s2)
    }
  }

  # Convert object
  dateformat <- as.Date(dateformat, "%d%m%Y")
  dateformat
}


#' Creates `cache_dir`
#'
#' @param cache_dir a directory path
#' @param verbose logical, display parameters
#' @param suffix a suffix
#'
#' @noRd
bde_hlp_cachedir <- function(cache_dir = NULL, verbose = FALSE, suffix = NULL) {
  # Check cache dir if is null
  if (is.null(cache_dir)) {
    # Check if set via options
    cache_dir <- getOption("bde_cache_dir", NULL)

    if (is.null(cache_dir)) {
      # Not set - using tempdir
      cache_dir <- tempdir()

      if (!is.null(suffix)) {
        cache_dir <- file.path(cache_dir, suffix)
      }

      if (verbose) {
        message("tidyBdE> Caching on temporary directory ", cache_dir)
      }
      return(cache_dir)
    } else {
      # Set via options
      if (verbose) {
        message("tidyBdE> Cache dir detected on options: ", cache_dir)
      }
    }
  }

  # When provided
  if (!is.null(suffix)) {
    cache_dir <- file.path(gsub(file.path("", suffix), "", cache_dir), suffix)
  }

  if (dir.exists(cache_dir)) {
    if (verbose) {
      message("tidyBdE> Cache dir is ", cache_dir)
    }
    return(cache_dir)
  }

  dir.create(cache_dir, recursive = TRUE)
  if (verbose) {
    message("tidyBdE> Cache dir created on ", cache_dir)
  }
  cache_dir
}

#' Helper for downloading files
#'
#' @param url resource url
#'
#' @param local_file local file to be created
#'
#' @param verbose logical, display parameters and messages
#'
#' @noRd
bde_hlp_download <- function(url, local_file, verbose) {
  if (verbose) {
    message("tidyBdE> Downloading file from ", url, "\n\n")
  }

  err_dwnload <- tryCatch(
    download.file(url, local_file, quiet = isFALSE(verbose), mode = "wb"),
    # nocov start
    warning = function(e) {
      TRUE
    }
  )
  # nocov end
  # Try again if not working
  # This time display a message

  # nocov start
  if (isTRUE(err_dwnload)) {
    if (verbose) {
      message("tidyBdE> Trying again")
    }

    err_dwnload <- tryCatch(
      download.file(url, local_file, quiet = isFALSE(verbose), mode = "wb"),
      # nocov start
      warning = function(e) {
        message(
          "tidyBdE> URL \n ",
          url,
          "\nnot reachable.\n\n",
          "If you think this is a bug consider opening an issue"
        )
        TRUE
      }
    )
  }
  # nocov end

  # On warning stop the execution
  if (isTRUE(err_dwnload)) {
    return(FALSE)
    # nocov end
  }
  TRUE
}


#' Guess formats
#'
#' @param tbl a tibble
#' @param preserve vector of names to preserve
#' @noRd
bde_hlp_guess <- function(tbl, preserve = "") {
  for (i in names(tbl)) {
    if (class(tbl[[i]])[1] == "character" && !(i %in% preserve)) {
      tbl[i] <- readr::parse_guess(
        tbl[[i]],
        locale = readr::locale(
          grouping_mark = "",
          decimal_mark = "."
        ),
        na = c("_", "...")
      )
    }
  }
  tbl
}


#' To chars
#'
#' @param tbl a tibble
#' @param preserve vector of names to preserve
#' @noRd
bde_hlp_tochar <- function(tbl, preserve = "") {
  for (i in names(tbl)) {
    if (class(tbl[[i]])[1] != "character" && !(i %in% preserve)) {
      tbl[i] <- as.character(tbl[[i]])
    }
  }
  tbl
}


#' To double
#'
#' @param tbl a tibble
#' @param preserve vector of names to preserve
#' @noRd
bde_hlp_todouble <- function(tbl, preserve = "") {
  for (i in names(tbl)) {
    if (class(tbl[[i]])[1] == "character" && !(i %in% preserve)) {
      tbl[i] <-
        readr::parse_double(
          tbl[[i]],
          locale = readr::locale(
            grouping_mark = "",
            decimal_mark = "."
          ),
          na = c("_", "...")
        )
    }
  }
  tbl
}


#' Return empty tibble
#' @return a tibble.
#'
#' @examples
#'
#' bde_hlp_return_null()
#' @noRd
bde_hlp_return_null <- function(msg = "Offline. Returning an empty tibble") {
  # nocov start
  message(paste0("tidyBdE> ", msg))
  tbl <- tibble::tibble(x = NULL)
  tbl
  # nocov end
}

test_that("Parse dates", {
  # Test bde_parse_dates----
  testdates <-
    c("2002", "MAR 2002", "01 MAR 2002", "---", NA, "-an,a")
  parsed <- bde_parse_dates(testdates)

  expect_s3_class(parsed, "Date")

  would_parse <- c(
    "02 FEB2019",
    "02 DIC 2019",
    "MAR 2020",
    "ENE2020",
    "2020",
    "12-1993",
    "01-02-2014",
    "31/12/2009"
  )

  parsed_ok <- bde_parse_dates(would_parse)
  expect_false(anyNA(parsed_ok))
})

#' Relevant Indicators of Spain
#'
#' @description
#' Set of helper functions for downloading some of the most relevant
#' macroeconomic indicators of Spain. Metadata available in [bde_ind_db].
#'
#' @rdname bde_indicators
#' @name bde_indicators
#'
#' @export
#'
#' @family indicators
#'
#' @encoding UTF-8
#'
#' @inheritParams bde_series_load
#'
#' @inheritDotParams bde_series_load -series_code
#'
#' @return A [tibble][tibble::tbl_df] with the required series.
#'
#' @seealso [bde_series_load()], [bde_catalog_search()]
#'
#' @details
#' These functions are convenient wrappers around [bde_series_load()] for
#' specific series. Use `verbose = TRUE, extract_metadata = TRUE` to see the
#' specification and the source.
#'
#' @examplesIf bde_check_access()
#' \donttest{
#' bde_ind_gdp_var()
#' }
bde_ind_gdp_var <- function(series_label = "GDP_YoY", ...) {
  db <- tidyBdE::bde_ind_db
  seq_num <- db[db$tidyBdE_fun == "bde_ind_gdp_var", "Numero_secuencial"]
  seq_num <- as.character(seq_num)

  econom_ind <-
    bde_series_load(seq_num, series_label = series_label, ...)
  econom_ind <- econom_ind[!is.na(econom_ind[[2]]), ]

  econom_ind
}

#' @rdname bde_indicators
#'
#' @export
bde_ind_unemployment_rate <-
  function(series_label = "Unemployment_Rate", ...) {
    db <- tidyBdE::bde_ind_db
    seq_num <- db[
      db$tidyBdE_fun == "bde_ind_unemployment_rate",
      "Numero_secuencial"
    ]
    seq_num <- as.character(seq_num)

    econom_ind <- bde_series_load(seq_num, series_label = series_label, ...)
    econom_ind <- econom_ind[!is.na(econom_ind[[2]]), ]

    econom_ind
  }

#' @rdname bde_indicators
#'
#' @export
bde_ind_euribor_12m_monthly <-
  function(series_label = "Euribor_12M_Monthly", ...) {
    db <- tidyBdE::bde_ind_db
    seq_num <- db[
      db$tidyBdE_fun == "bde_ind_euribor_12m_monthly",
      "Numero_secuencial"
    ]
    seq_num <- as.character(seq_num)

    econom_ind <- bde_series_load(seq_num, series_label = series_label, ...)
    econom_ind <- econom_ind[!is.na(econom_ind[[2]]), ]

    econom_ind
  }

#' @rdname bde_indicators
#'
#' @export
bde_ind_euribor_12m_daily <-
  function(series_label = "Euribor_12M_Daily", ...) {
    db <- tidyBdE::bde_ind_db
    seq_num <- db[
      db$tidyBdE_fun == "bde_ind_euribor_12m_daily",
      "Numero_secuencial"
    ]
    seq_num <- as.character(seq_num)

    econom_ind <- bde_series_load(seq_num, series_label = series_label, ...)
    econom_ind <- econom_ind[!is.na(econom_ind[[2]]), ]

    econom_ind
  }

#' @rdname bde_indicators
#'
#' @export
bde_ind_cpi_var <-
  function(series_label = "Consumer_price_index_YoY", ...) {
    db <- tidyBdE::bde_ind_db
    seq_num <- db[
      db$tidyBdE_fun == "bde_ind_cpi_var",
      "Numero_secuencial"
    ]
    seq_num <- as.character(seq_num)

    econom_ind <- bde_series_load(seq_num, series_label = series_label, ...)
    econom_ind <- econom_ind[!is.na(econom_ind[[2]]), ]

    econom_ind
  }

#' @rdname bde_indicators
#'
#' @export
bde_ind_ibex_monthly <- function(series_label = "IBEX_index_month", ...) {
  db <- tidyBdE::bde_ind_db
  seq_num <- db[
    db$tidyBdE_fun == "bde_ind_ibex_monthly",
    "Numero_secuencial"
  ]
  seq_num <- as.character(seq_num)

  econom_ind <- bde_series_load(seq_num, series_label = series_label, ...)
  econom_ind <- econom_ind[!is.na(econom_ind[[2]]), ]

  econom_ind
}

#' @rdname bde_indicators
#'
#' @export
bde_ind_ibex_daily <- function(series_label = "IBEX_index_day", ...) {
  db <- tidyBdE::bde_ind_db
  seq_num <- db[
    db$tidyBdE_fun == "bde_ind_ibex_daily",
    "Numero_secuencial"
  ]
  seq_num <- as.character(seq_num)

  econom_ind <- bde_series_load(seq_num, series_label = series_label, ...)
  econom_ind <- econom_ind[!is.na(econom_ind[[2]]), ]

  econom_ind
}

#' @rdname bde_indicators
#'
#' @export
bde_ind_gdp_quarterly <- function(series_label = "GDP_quarterly_value", ...) {
  db <- tidyBdE::bde_ind_db
  seq_num <- db[
    db$tidyBdE_fun == "bde_ind_gdp_quarterly",
    "Numero_secuencial"
  ]
  seq_num <- as.character(seq_num)

  econom_ind <- bde_series_load(seq_num, series_label = series_label, ...)
  econom_ind <- econom_ind[!is.na(econom_ind[[2]]), ]

  econom_ind
}


#' @rdname bde_indicators
#'
#' @export
bde_ind_population <- function(series_label = "Population_Spain", ...) {
  db <- tidyBdE::bde_ind_db
  seq_num <- db[
    db$tidyBdE_fun == "bde_ind_population",
    "Numero_secuencial"
  ]
  seq_num <- as.character(seq_num)

  econom_ind <- bde_series_load(seq_num, series_label = series_label, ...)
  econom_ind <- econom_ind[!is.na(econom_ind[[2]]), ]

  econom_ind
}

#' @export
#' @rdname bde_indicators
#' @usage NULL
bde_ind_ibex <- bde_ind_ibex_monthly

test_that("Indicators", {
  skip_on_cran()
  skip_if_bde_offline()

  # Test indicators----
  n <- expect_silent(bde_ind_gdp_var())
  expect_true(nrow(n) > 10)

  n2 <- expect_silent(bde_ind_unemployment_rate())
  expect_true(nrow(n2) > 10)

  n3 <- expect_silent(bde_ind_euribor_12m_monthly())
  expect_true(nrow(n3) > 10)

  n4 <- expect_silent(bde_ind_euribor_12m_daily())
  expect_true(nrow(n4) > 10)

  n5 <- expect_silent(bde_ind_cpi_var())
  expect_true(nrow(n5) > 10)

  n6 <- expect_silent(bde_ind_ibex())
  expect_true(nrow(n6) > 10)

  n6b <- expect_silent(bde_ind_ibex_monthly())
  expect_identical(n6, n6b)

  n7 <- expect_silent(bde_ind_gdp_quarterly())
  expect_true(nrow(n7) > 10)

  n8 <- expect_silent(bde_ind_population())
  expect_true(nrow(n8) > 10)

  n9 <- expect_silent(bde_ind_ibex_daily())
  expect_true(nrow(n9) > 10)
})

#' BdE color palettes
#'
#' Custom palettes based on the publications of BdE. These are manual
#' palettes with a maximum of 6 colors.
#'
#' @family bde_plot
#'
#' @return A vector of colors.
#'
#' @export
#' @param n The number of colors (`>= 1`) in the palette.
#' @param palette A valid palette name.
#' @param alpha An alpha-transparency level in the range `[0,1]` (`0` means
#'   transparent and `1` means opaque). A missing, i.e., `alpha = NULL`, does
#'   not add opacity codes (`"FF"`) to the individual color hex codes.
#'   See [ggplot2::alpha()].
#' @param rev Logical indicating whether the ordering of the colors should be
#'   reversed.
#' @examples
#'
#' # BdE vivid pal
#' scales::show_col(bde_tidy_palettes(palette = "bde_vivid_pal"),
#'   labels = FALSE
#' )
#'
#' # BdE rose pal
#' scales::show_col(bde_tidy_palettes(palette = "bde_rose_pal"),
#'   labels = FALSE
#' )
#'
#' # BdE qual pal
#' scales::show_col(bde_tidy_palettes(palette = "bde_qual_pal"),
#'   labels = FALSE
#' )
bde_tidy_palettes <- function(
  n = 6,
  palette = c(
    "bde_vivid_pal",
    "bde_rose_pal",
    "bde_qual_pal"
  ),
  alpha = NULL,
  rev = FALSE
) {
  palette <- match.arg(palette)

  cols <- switch(palette,
    "bde_vivid_pal" = c(
      "#4180C2",
      "#D86E7B",
      "#F89E63",
      "#5FBD6A",
      "#62C8D0",
      "#AC8771"
    ),
    "bde_rose_pal" = c(
      "#b7365c",
      "#cb6e8a",
      "#db9aad",
      "#0a50a1",
      "#5385bd",
      "#89AEDA"
    ),
    "bde_qual_pal" = c(
      "#b55b4a",
      "#2e76bc",
      "#fece64",
      "#68be57",
      "#858788",
      "#f9b4af"
    )
  )

  n_col <- length(cols)
  if (n > n_col) {
    message(
      "tidyBdE> ",
      palette,
      " has ",
      n_col,
      ", requested ",
      n,
      ". Returning ",
      n_col,
      " colors."
    )

    n <- n_col
  }

  endcols <- cols[seq_len(n)]

  # Options
  if (rev) {
    endcols <- rev(endcols)
  }
  if (!is.null(alpha)) {
    endcols <- ggplot2::alpha(endcols, alpha)
  }

  endcols
}

test_that("Test error", {
  expect_snapshot(bde_tidy_palettes(palette = "none"), error = TRUE)
})


test_that("Max value", {
  ok <- bde_tidy_palettes(n = 6)

  expect_snapshot(nmore <- bde_tidy_palettes(n = 23))

  expect_identical(ok, nmore)
})

test_that("Switch pal", {
  ok <- bde_tidy_palettes(n = 6)
  other <- bde_tidy_palettes(n = 6, palette = "bde_rose_pal")
  expect_false(any(ok == other))

  other2 <- bde_tidy_palettes(n = 6, palette = "bde_qual_pal")
  expect_false(any(ok == other2))
  expect_false(any(other == other2))
})


test_that("test n", {
  ok <- bde_tidy_palettes(n = 6)
  ok3 <- bde_tidy_palettes(n = 3)

  expect_identical(ok[seq_len(3)], ok3)
})


test_that("test rev", {
  ok <- bde_tidy_palettes(n = 6)
  ok3 <- bde_tidy_palettes(n = 3, rev = TRUE)

  expect_identical(ok[rev(seq_len(3))], ok3)
})

test_that("test alpha", {
  ok <- bde_tidy_palettes(n = 6)
  ok3 <- bde_tidy_palettes(n = 6, alpha = 0.2)

  expect_identical(ggplot2::alpha(ok, 0.2), ok3)
})

#' BdE superseded palettes
#'
#' @description
#' `r lifecycle::badge('superseded')`
#'
#' These functions have been superseded; see [bde_tidy_palettes()] as a
#' replacement.
#'
#' Custom palettes based on the publications of BdE.
#'
#' @return A palette of colors.
#'
#' @param ... Further arguments of the functions.
#'
#' @name bde_vivid_pal
#' @rdname bde_pals
#'
#' @export
#' @keywords internal
#' @examples
#'
#' # BdE vivid pal
#' scales::show_col(bde_vivid_pal()(6), labels = FALSE)
#'
#' # BdE rose pal
#' scales::show_col(bde_rose_pal()(6), labels = FALSE)
bde_vivid_pal <- function(...) {
  if (requireNamespace("lifecycle", quietly = TRUE)) {
    lifecycle::deprecate_soft(
      "0.3.5",
      "bde_vivid_pal()",
      "bde_tidy_palettes()"
    )
  }

  pal <- c("#4180C2", "#D86E7B", "#F89E63", "#5FBD6A", "#62C8D0", "#AC8771")
  scales::manual_pal(pal)
}

#' @rdname bde_pals
#' @keywords internal
#' @export
bde_rose_pal <- function(...) {
  if (requireNamespace("lifecycle", quietly = TRUE)) {
    lifecycle::deprecate_soft(
      "0.3.5",
      "bde_rose_pal()",
      "bde_tidy_palettes()"
    )
  }
  pal <- c("#b7365c", "#cb6e8a", "#db9aad", "#0a50a1", "#5385bd", "#89AEDA")
  scales::manual_pal(pal)
}

test_that("bde_vivid_pal", {
  skip_if_not_installed("lifecycle")
  expect_snapshot(vpal <- bde_vivid_pal()(3))

  expect_identical(
    bde_tidy_palettes(n = 3, "bde_vivid_pal"),
    vpal
  )
})

test_that("bde_rose_pal", {
  skip_if_not_installed("lifecycle")
  expect_snapshot(vpal <- bde_rose_pal()(4))

  expect_identical(
    bde_tidy_palettes(n = 4, "bde_rose_pal"),
    vpal
  )
})

#' Database of selected macroeconomic indicators
#'
#' @description
#' Minimal metadata of the selected macroeconomic indicators included in helper
#' functions of \CRANpkg{tidyBdE} (see [bde_indicators]). Full metadata can be
#' accessed via [bde_catalog_load()]
#'
#' @docType data
#'
#' @family indicators
#'
#' @encoding UTF-8
#'
#' @name bde_ind_db
#'
#' @format
#' A [tibble][tibble::tbl_df] of `r nrow(bde_ind_db)` rows and
#' `r ncol(bde_ind_db)` columns with the following fields:
#'
#' \describe{
#'   \item{tidyBdE_fun}{Function name, see [bde_indicators].}
#'   \item{Numero_secuencial}{Series code, see [bde_series_load()].}
#'   \item{Descripcion_de_la_serie}{Description of the series in Spanish.}
#'   \item{Fecha_de_la_primera_observacion}{Starting date of the indicator.}
#'   \item{Fecha_de_la_ultima_observacion}{Most recent date available.}
#'   \item{Fuente}{Data source.}
#' }
#'
#' @details
#'
#' ```{r child = "man/chunks/bde_ind_db_meta.Rmd"}
#' ```
#'
#' @examples
#' data("bde_ind_db")
#' bde_ind_db
#'
NULL

#' Load a single BdE time series
#'
#' Load a single time series provided by BdE.
#'
#' @export
#'
#' @family series
#'
#' @encoding UTF-8
#'
#' @param series_code A numeric value (or coercible with [base::as.double()])
#'   or vector of time series code(s), as defined in the field
#'   `Número secuencial` of the corresponding series. See [bde_catalog_load()].
#'
#' @param series_label Optional. Character vector or value. Allows specifying a
#'   custom label for the series extracted.
#'
#' @param out_format Whether the format should be returned as "long" or "wide".
#'   Possible values are `"wide"` or `"long"`. See **Value** for details and
#'   section **Examples**.
#' @inheritParams bde_series_full_load
#'
#' @return
#' A [tibble][tibble::tbl_df] with a field `Date`:
#' - With `out_format = "wide"`, each series is presented in a separate
#'   column with the name defined by `series_label`.
#' - With `out_format = "long"`, the tibble has two additional columns:
#'   - `serie_name` with the label of each series.
#'   - `serie_value` with the corresponding value.
#'
#' `"wide"` format is more suitable for exporting to a `.csv` file while
#' `"long"` format is more suitable for creating plots with
#' [ggplot2::ggplot()]. See also [tidyr::pivot_longer()] and
#' [tidyr::pivot_wider()].
#'
#' @description
#'
#' The series alias is a positional code showing the location (column and/or
#' row) of the series in the table. However, although it is unique, it is not a
#' good candidate to be used as the series ID, as it is subject to change. If a
#' series changes position in the table, its alias will also change.
#'
#' To ensure series can still be identified, even after these changes, they are
#' assigned a sequential number (referred to as `series_code` in this function).
#'
#' Note that a single series may appear in different tables, so it can have
#' several aliases. If you need to search by alias, use
#' [bde_series_full_load()].
#'
#' @note
#' This function attempts to coerce the columns to numbers. For some series, a
#' warning may be displayed if the parsing fails.
#'
#' @seealso [bde_catalog_load()],
#' [bde_catalog_search()], [bde_indicators()]
#'
#' @examplesIf bde_check_access()
#' \donttest{
#' # Metadata
#' bde_series_load(573234, verbose = TRUE, extract_metadata = TRUE)
#'
#' # Data
#' bde_series_load(573234, extract_metadata = FALSE)
#'
#' # Vectorized
#' bde_series_load(c(573234, 573214),
#'   series_label = c("US/EUR", "GBP/EUR"),
#'   extract_metadata = TRUE
#' )
#'
#' wide <- bde_series_load(c(573234, 573214),
#'   series_label = c("US/EUR", "GBP/EUR")
#' )
#'
#' # Wide format
#' wide
#'
#' # Long format
#' long <- bde_series_load(c(573234, 573214),
#'   series_label = c("US/EUR", "GBP/EUR"),
#'   out_format = "long"
#' )
#'
#' long
#'
#' # Use with ggplot
#' library(ggplot2)
#'
#' ggplot(long, aes(Date, serie_value)) +
#'   geom_line(aes(group = serie_name, color = serie_name)) +
#'   scale_color_bde_d() +
#'   theme_tidybde()
#' }
bde_series_load <- function(
  series_code,
  series_label = NULL,
  out_format = "wide",
  parse_dates = TRUE,
  parse_numeric = TRUE,
  cache_dir = NULL,
  update_cache = FALSE,
  verbose = FALSE,
  extract_metadata = FALSE
) {
  if (missing(series_code)) {
    stop("`series_code` can't be NULL")
  }

  series_code <- as.double(series_code)
  # Remove NAs
  series_code <- series_code[!is.na(series_code)]

  if (is.null(series_label)) {
    series_label <- as.character(series_code)
  }
  if (anyNA(series_label)) {
    stop("`series_label` should not contain NA values")
  }

  series_label <- unique(as.character(series_label))

  if (length(series_code) != length(series_label)) {
    stop("`series_label` and `series_code` should have the same length")
  }

  # Lookup on catalogs
  all_catalogs <- bde_catalog_load(
    catalog = "ALL",
    parse_dates = parse_dates,
    cache_dir = cache_dir,
    update_cache = update_cache,
    verbose = verbose
  )

  # nocov start
  if (nrow(all_catalogs) == 0) {
    tbl <- bde_hlp_return_null()
    return(tbl)
  }
  # nocov end

  all_catalogs <- all_catalogs[!is.na(all_catalogs[[2]]), c(2, 3, 4)]

  df_list <- lapply(series_code, function(x) {
    if (verbose) {
      message("tidyBdE> Extracting series ", x, "\n\n")
    }

    # Select file

    csv_file <- all_catalogs[all_catalogs[[1]] == x, c(2, 3)]

    if (nrow(csv_file) == 0) {
      message("`series_code` not found on catalogs.")
      tbl <- bde_hlp_return_null()
      return(tbl)
    }

    # Select first record
    csv_file_name <- as.character(csv_file[1, 2])
    alias_serie <- as.character(csv_file[1, 1])

    if (verbose) {
      message(
        "tidyBdE> Downloading serie ",
        x,
        " from file ",
        csv_file_name,
        " (alias ",
        alias_serie,
        ")."
      )
    }

    # Download and select series
    serie_file <- bde_series_full_load(
      csv_file_name,
      parse_dates = parse_dates,
      parse_numeric = parse_numeric,
      cache_dir = cache_dir,
      update_cache = update_cache,
      verbose = verbose,
      extract_metadata = extract_metadata
    )

    # nocov start
    if (nrow(serie_file) == 0) {
      tbl <- bde_hlp_return_null()
      return(tbl)
    }
    if (!(alias_serie %in% names(serie_file))) {
      if (verbose) {
        message(
          "tidyBdE> ",
          "Serie with alias '",
          alias_serie,
          "' not available on ",
          csv_file_name,
          ". "
        )
      }

      # Prevent deprecation

      return(bde_hlp_return_null())

      # nocov end
    } else {
      serie_file <- serie_file[c("Date", alias_serie)]
    }

    i <- match(x, series_code)
    names(serie_file) <- c("Date", "serie_value")
    serie_file$serie_name <- as.character(series_label[i])
    # Rearrange
    serie_file <- serie_file[c("Date", "serie_name", "serie_value")]

    serie_file
  })

  # Clean
  nrows <- unlist(lapply(df_list, nrow)) > 0

  df_list <- df_list[nrows]

  # Check that all dfs have Date field

  has_date <- vapply(
    df_list,
    function(x) {
      "Date" %in% names(x)
    },
    FUN.VALUE = logical(1)
  )

  df_list <- df_list[has_date]

  # Check number of series and merge (if needed)
  if (length(df_list) == 0) {
    return(bde_hlp_return_null())
  }

  # To long
  end <- dplyr::bind_rows(df_list)
  # As factors
  end$serie_name <- factor(end$serie_name, levels = unique(end$serie_name))
  # Factors

  if (out_format == "wide" || isTRUE(extract_metadata)) {
    end <- tidyr::pivot_wider(
      end,
      id_cols = "Date",
      names_from = "serie_name",
      values_from = "serie_value"
    )
  }

  end <- tibble::as_tibble(end)
  end
}


#' Load BdE full time series files
#'
#' Load a full time series file provided by BdE.
#'
#' ## About BdE file naming
#'
#' The series name is a positional code showing the location of the table. For
#' example, table **be_6_1** represents the Table 1, Chapter 6 of the
#' Statistical Bulletin ("BE"). Although it is a unique value, it is subject
#' to change (i.e. a new table is inserted before).
#'
#' For that reason, the function [bde_series_load()] is more suitable for
#' extracting specific time series.
#'
#' @export
#'
#' @family series
#'
#' @encoding UTF-8
#'
#' @param series_csv csv file of a series, as defined in the field
#'   `Nombre del archivo con los valores de la serie` of the corresponding
#'   catalog. See [bde_catalog_load()].
#'
#' @inheritParams bde_catalog_load
#'
#' @param parse_numeric Logical. If `TRUE` the columns would be parsed to
#'   double (numeric) values. See **Note**.
#'
#' @param extract_metadata Logical `TRUE/FALSE`. On `TRUE` the output is the
#'   metadata of the requested series.
#'
#' @return
#' A [tibble][tibble::tbl_df] with a field `Date` and the alias of the fields
#' series as described on the catalogs. See [bde_catalog_load()].
#'
#' @note
#' This function tries to coerce the columns to numbers. For some series a
#' warning may be displayed if the parser fails. You can override the default
#' behavior with `parse_numeric = FALSE`
#'
#' @examplesIf bde_check_access()
#' \donttest{
#' # Metadata
#' bde_series_full_load("TI_1_1.csv", extract_metadata = TRUE)
#'
#' # Data
#' bde_series_full_load("TI_1_1.csv")
#' }
bde_series_full_load <- function(
  series_csv,
  parse_dates = TRUE,
  parse_numeric = TRUE,
  cache_dir = NULL,
  update_cache = FALSE,
  verbose = FALSE,
  extract_metadata = FALSE
) {
  stopifnot(
    is.null(cache_dir) || is.character(cache_dir),
    is.logical(verbose),
    is.logical(parse_dates),
    is.logical(update_cache)
  )

  if (length(grep(".csv", series_csv)) == 0) {
    series_csv <- paste0(series_csv, ".csv")
  }

  pp <- substr(series_csv, 1, 2)

  # Get cache dir
  cache_dir <- bde_hlp_cachedir(
    cache_dir = cache_dir,
    verbose = verbose,
    suffix = pp
  )

  # Create if not exist
  if (!dir.exists(cache_dir)) {
    dir.create(cache_dir, recursive = TRUE)
  }

  base_url <- paste0(
    "https://www.bde.es/webbe/es/estadisticas/",
    "compartido/datos/csv/"
  )

  serie_file <- tolower(series_csv)

  full_url <- paste0(base_url, serie_file)
  local_file <- file.path(cache_dir, serie_file)

  # If no serie is found, update
  if (update_cache || isFALSE(file.exists(local_file))) {
    if (!bde_check_access()) {
      tbl <- bde_hlp_return_null()
      return(tbl)
    }

    result <- bde_hlp_download(
      url = full_url,
      local_file = local_file,
      verbose = verbose
    )

    if (isFALSE(result)) {
      # Clean up the file if it was produced. Is not valid
      file_full_path <- path.expand(local_file)
      if (file.exists(file_full_path)) {
        unlink(file_full_path, force = TRUE, recursive = TRUE)
      }
      return(invisible())
    }
  } else {
    if (verbose) {
      message("tidyBdE> Reading file ", serie_file, " from cache.")
    }
  }

  # Catch error
  # nocov start
  r <- readLines(local_file)
  if (length(r) == 0) {
    message("File ", local_file, " not valid")
    return(invisible())
  }
  # nocov end

  # Serie load
  serie_load <- read.csv2(
    local_file,
    sep = ",",
    stringsAsFactors = FALSE,
    na.strings = "",
    header = FALSE,
    fileEncoding = "latin1"
  )

  serie_load <- tibble::as_tibble(serie_load)

  # Always in third line
  newnames <- as.character(serie_load[3, ])
  newnames[1] <- "Date"

  names(serie_load) <- newnames

  # Metadata
  # Always lines 1 to 6
  meta_serie <- serie_load[seq_len(6), ]

  # Add FUENTE and NOTAS
  source_notes <-
    serie_load[serie_load[[1]] %in% c("FUENTE", "NOTAS"), ]

  if (extract_metadata) {
    return(meta_serie)
  }

  meta_serie <- dplyr::bind_rows(meta_serie, source_notes)

  # Data: the rest of lines
  data_serie <- serie_load[-seq_len(6), ]

  data_serie <-
    data_serie[!toupper(data_serie$Date) %in% c("FUENTE", "NOTAS"), ]

  newnames_data <- as.character(meta_serie[4, ])
  newnames_data[1] <- "Date"

  # Parse dates
  if (parse_dates) {
    if (verbose) {
      message("tidyBdE> Parsing dates")
    }
    date_fields <- names(data_serie)[grep(
      "Date",
      names(data_serie),
      fixed = TRUE
    )]

    for (i in seq_along(date_fields)) {
      field <- date_fields[i]

      data_serie[field] <- bde_parse_dates(data_serie[[field]])
    }
  }

  if (parse_numeric) {
    if (verbose) {
      message("tidyBdE> Parsing fields to double")
    }
    # Fields to double
    data_serie <- bde_hlp_todouble(data_serie, preserve = "Date")
  }
  data_serie
}

test_that("Indicators", {
  expect_error(bde_series_load())

  skip_on_cran()
  skip_if_bde_offline()

  # Test load series all----
  expect_null(bde_series_full_load("aa"))

  expect_message(bde_series_full_load(
    "TI_1_1.csv",
    cache_dir = tempdir(),
    verbose = TRUE
  ))
  expect_message(bde_series_full_load(
    "TI_1_1.csv",
    cache_dir = NULL,
    verbose = TRUE
  ))
  expect_message(bde_series_full_load(
    "CF0101.csv",
    cache_dir = NULL,
    verbose = TRUE
  ))
  expect_silent(bde_series_full_load("CF0101"))

  data <- bde_series_full_load("TI_1_1.csv")
  meta <-
    bde_series_full_load("TI_1_1.csv", extract_metadata = TRUE)

  expect_true(nrow(data) > nrow(meta))

  # Test load series ----
  expect_warning(bde_series_load("aa"))
  expect_identical(bde_series_load(12345678910), bde_hlp_return_null())
  expect_error(bde_series_load(c(573234, 573214), series_label = c(1, NA)))
  expect_error(bde_series_load(c(573234, 573214), series_label = c("1", "1")))
  expect_error(bde_series_load(573234, series_label = c("a", "b")))

  expect_silent(bde_series_load(c(573234, 573214), series_label = c("a", "b")))

  expect_silent(bde_series_load(573234, series_label = "a"))
  expect_silent(bde_series_load("573234", series_label = "a"))
  expect_warning(bde_series_load(c("573234", "a")))
  expect_silent(bde_series_load(573234, series_label = NULL))
  expect_silent(bde_series_load(573234, extract_metadata = TRUE))
  expect_message(bde_series_load(573234, verbose = TRUE))

  meta <- bde_series_load(573234, extract_metadata = TRUE)
  data <- bde_series_load(573234, extract_metadata = FALSE)
  expect_true(nrow(data) > nrow(meta))

  # Test long and wide
  wide <- bde_series_load(c(573234, 573214), series_label = c("a", "b"))
  long <- bde_series_load(
    c(573234, 573214),
    series_label = c("a", "b"),
    out_format = "long"
  )

  expect_equal(ncol(long), 3)
  expect_true(nrow(long) > nrow(wide))
  expect_equal(class(long$serie_name), "factor")
  expect_equal(levels(long$serie_name), names(wide[, -1]))

  # Wide and long does not affect on metadata
  wide <- bde_series_load(
    c(573234, 573214),
    series_label = c("a", "b"),
    extract_metadata = TRUE
  )
  long <- bde_series_load(
    c(573234, 573214),
    series_label = c("a", "b"),
    out_format = "long",
    extract_metadata = TRUE
  )
  expect_identical(wide, long)
})

test_that("Series full", {
  skip_on_cran()
  skip_if_bde_offline()
  dir <- file.path(tempdir(), paste0("testoff", sample.int(10, 1)))

  # Test all offline

  # Get a series
  tc_catalog <- bde_catalog_load("TI", cache_dir = dir)

  all_tables <- tc_catalog$Nombre_del_archivo_con_los_valores_de_la_serie
  all_names <- as.character(unique(all_tables))

  full_1 <- bde_series_full_load(all_names[1], cache_dir = dir)

  expect_s3_class(full_1, "data.frame")
  expect_gt(nrow(full_1), 5)

  options(bde_test_offline = TRUE)
  # Can't download series
  expect_message(bde_series_full_load(all_names[2], cache_dir = dir))

  fail <- bde_series_full_load(all_names[2], cache_dir = dir)
  expect_equal(nrow(fail), 0)

  # But can reload cached series
  expect_silent(bde_series_full_load(all_names[1], cache_dir = dir))
  full_2 <- bde_series_full_load(all_names[1], cache_dir = dir)

  expect_identical(full_1, full_2)

  # Now try online
  options(bde_test_offline = FALSE)

  failfix <- bde_series_full_load(all_names[2], cache_dir = dir)
  expect_gt(nrow(failfix), 10)
})

---
title: "Get started with tidyBdE"
description: Introduction to tidyBdE.
tbl-cap-location: bottom
vignette: >
  %\VignetteIndexEntry{Get started with tidyBdE}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

<!-- tidyBdE.qmd is generated from tidyBdE.qmd.orig. Please edit that file -->



**tidyBdE** is an **R** package that retrieves data from [Banco de
España](https://www.bde.es/webbe/en/estadisticas/recursos/descargas-completas.html).
Data are returned as a [tibble](https://tibble.tidyverse.org/), and the
package automatically detects the format of each time series (dates, characters
and numbers).

## Search series

Banco de España (**BdE**) provides several time series, either produced by the
institution itself or compiled from other sources, such as
[Eurostat](https://ec.europa.eu/eurostat) or [INE](https://www.ine.es/).

The basic entry point for searching time series is the catalog. You can search
for series by name:


``` r
library(tidyBdE)

library(ggplot2)
library(dplyr)
library(tidyr)

# Search for GBP in the "TC" (exchange rate) catalog.
xr_gbp <- bde_catalog_search("GBP", catalog = "TC")

xr_gbp |>
  select(Numero_secuencial, Descripcion_de_la_serie) |>
  # Display the table in the document.
  knitr::kable()
```

::: {#tbl-search}


| Numero_secuencial|Descripcion_de_la_serie                                            |
|-----------------:|:------------------------------------------------------------------|
|            573214|Tipo de cambio. Libras esterlinas por euro (GBP/EUR).Datos diarios |



Search results
:::

**Note:** BdE metadata is currently provided in Spanish only, so search terms
must be provided in Spanish to retrieve results. The institution is working on
an English version.

After finding a series, load the GBP/EUR exchange rate using the sequential
number reference (`Numero_secuencial`):


``` r
seq_number <- xr_gbp |>
  # Select the first record.
  slice(1) |>
  # Get the series ID.
  pull(Numero_secuencial) |>
  # Convert to numeric.
  as.double()

seq_number
#> [1] 573214

time_series <- bde_series_load(seq_number, series_label = "EUR_GBP_XR") |>
  filter(Date >= "2010-01-01" & Date <= "2020-12-31") |>
  drop_na()

time_series
#> # A tibble: 2,816 × 2
#>    Date       EUR_GBP_XR
#>    <date>          <dbl>
#>  1 2010-01-04      0.891
#>  2 2010-01-05      0.900
#>  3 2010-01-06      0.899
#>  4 2010-01-07      0.900
#>  5 2010-01-08      0.893
#>  6 2010-01-11      0.899
#>  7 2010-01-12      0.897
#>  8 2010-01-13      0.895
#>  9 2010-01-14      0.890
#> 10 2010-01-15      0.881
#> # ℹ 2,806 more rows
```

## Plot series

The package also provides a custom **ggplot2** theme based on BdE
publications:


``` r
ggplot(time_series, aes(x = Date, y = EUR_GBP_XR)) +
  geom_line(colour = bde_tidy_palettes(n = 1)) +
  geom_smooth(method = "gam", colour = bde_tidy_palettes(n = 2)[2]) +
  labs(
    title = "EUR/GBP Exchange Rate (2010-2020)",
    subtitle = "%",
    caption = "Source: BdE"
  ) +
  geom_vline(
    xintercept = as.Date("2016-06-23"),
    linetype = "dotted"
  ) +
  geom_label(aes(
    x = as.Date("2016-06-23"),
    y = 0.95,
    label = "Brexit"
  )) +
  coord_cartesian(ylim = c(0.7, 1)) +
  theme_tidybde()
```

<div class="figure">
<img src="./chart-1.png" alt="Figure 1: EUR/GBP Exchange Rate (2010-2020)" width="100%" />
<p class="caption">Figure 1: EUR/GBP Exchange Rate (2010-2020)</p>
</div>

The package also provides convenience functions for selected macroeconomic
series, so you do not need to search manually:


``` r
# Data in long format.

plotseries <- bde_ind_gdp_var("GDP YoY", out_format = "long") |>
  bind_rows(
    bde_ind_unemployment_rate("Unemployment Rate", out_format = "long")
  ) |>
  drop_na() |>
  filter(Date >= "2010-01-01" & Date <= "2019-12-31")

ggplot(plotseries, aes(x = Date, y = serie_value)) +
  geom_line(aes(color = serie_name), linewidth = 1) +
  labs(
    title = "Spanish Economic Indicators (2010-2019)",
    subtitle = "%",
    caption = "Source: BdE"
  ) +
  theme_tidybde() +
  scale_color_bde_d(palette = "bde_vivid_pal") # Use a custom package palette.
```

<div class="figure">
<img src="./macroseries-1.png" alt="Figure 2: Spanish Economic Indicators (2010-2019)" width="100%" />
<p class="caption">Figure 2: Spanish Economic Indicators (2010-2019)</p>
</div>

## A note on caching

You can use **tidyBdE** to create a local cache by setting the following
option:


``` r
options(bde_cache_dir = "./path/to/location")
```

When this option is set, **tidyBdE** looks for cached files in the
`bde_cache_dir` directory and loads them, speeding up data retrieval.

You can update the data after monthly or quarterly releases with the following
commands:


``` r
bde_catalog_update()

# Or use `update_cache = TRUE` in most functions.

bde_series_load("SOME ID", update_cache = TRUE)
```

# Test error

    Code
      bde_tidy_palettes(palette = "none")
    Condition
      Error in `match.arg()`:
      ! 'arg' should be one of "bde_vivid_pal", "bde_rose_pal", "bde_qual_pal"

# Max value

    Code
      nmore <- bde_tidy_palettes(n = 23)
    Message
      tidyBdE> bde_vivid_pal has 6, requested 23. Returning 6 colors.

# bde_vivid_pal

    Code
      vpal <- bde_vivid_pal()(3)
    Condition
      Warning:
      `bde_vivid_pal()` was deprecated in tidyBdE 0.3.5.
      i Please use `bde_tidy_palettes()` instead.

# bde_rose_pal

    Code
      vpal <- bde_rose_pal()(4)
    Condition
      Warning:
      `bde_rose_pal()` was deprecated in tidyBdE 0.3.5.
      i Please use `bde_tidy_palettes()` instead.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scales.R
\name{scales_bde}
\alias{scales_bde}
\alias{scale_color_bde_d}
\alias{scale_colour_bde_d}
\alias{scale_fill_bde_d}
\alias{scale_color_bde_c}
\alias{scale_colour_bde_c}
\alias{scale_fill_bde_c}
\title{BdE scales for \CRANpkg{ggplot2}}
\usage{
scale_color_bde_d(
  palette = c("bde_vivid_pal", "bde_rose_pal", "bde_qual_pal"),
  alpha = NULL,
  rev = FALSE,
  ...
)

scale_fill_bde_d(
  palette = c("bde_vivid_pal", "bde_rose_pal", "bde_qual_pal"),
  alpha = NULL,
  rev = FALSE,
  ...
)

scale_color_bde_c(
  palette = c("bde_rose_pal", "bde_vivid_pal", "bde_qual_pal"),
  alpha = NULL,
  rev = FALSE,
  guide = "colorbar",
  ...
)

scale_fill_bde_c(
  palette = c("bde_rose_pal", "bde_vivid_pal", "bde_qual_pal"),
  alpha = NULL,
  rev = FALSE,
  guide = "colorbar",
  ...
)
}
\arguments{
\item{palette}{Name of the BdE palette to apply. See \code{\link[=bde_tidy_palettes]{bde_tidy_palettes()}}
for details.}

\item{alpha}{An alpha-transparency level in the range \verb{[0,1]} (\code{0} means
transparent and \code{1} means opaque). A missing, i.e., \code{alpha = NULL}, does
not add opacity codes (\code{"FF"}) to the individual color hex codes.
See \code{\link[ggplot2:reexports]{ggplot2::alpha()}}.}

\item{rev}{Logical indicating whether the ordering of the colors should be
reversed.}

\item{...}{Further arguments of \code{\link[ggplot2:discrete_scale]{ggplot2::discrete_scale()}} or
\code{\link[ggplot2:continuous_scale]{ggplot2::continuous_scale()}}.}

\item{guide}{A function used to create a guide or its name. See
\code{\link[ggplot2:guides]{guides()}} for more information.}
}
\value{
A \CRANpkg{ggplot2} color scale.
}
\description{
Scales to be used with the \CRANpkg{ggplot2} package. Discrete palettes are
named \verb{scale_*_bde_d}, while continuous palettes are named \verb{scale_*_bde_c}.
}
\examples{
library(ggplot2)

set.seed(596)
txsamp <- subset(
  txhousing,
  city \%in\% c(
    "Houston", "Fort Worth",
    "San Antonio", "Dallas", "Austin"
  )
)

ggplot(txsamp, aes(x = sales, y = median)) +
  geom_point(aes(colour = city)) +
  scale_color_bde_d() +
  theme_minimal()

ggplot(txsamp, aes(x = sales, y = median)) +
  geom_point(aes(colour = city)) +
  scale_color_bde_d("bde_qual_pal") +
  theme_minimal()

}
\seealso{
\code{\link[ggplot2:discrete_scale]{ggplot2::discrete_scale()}}, \code{\link[ggplot2:continuous_scale]{ggplot2::continuous_scale()}}

Other bde_plot: 
\code{\link{bde_tidy_palettes}()},
\code{\link{theme_tidybde}()}
}
\concept{bde_plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/series.R
\encoding{UTF-8}
\name{bde_series_full_load}
\alias{bde_series_full_load}
\title{Load BdE full time series files}
\usage{
bde_series_full_load(
  series_csv,
  parse_dates = TRUE,
  parse_numeric = TRUE,
  cache_dir = NULL,
  update_cache = FALSE,
  verbose = FALSE,
  extract_metadata = FALSE
)
}
\arguments{
\item{series_csv}{csv file of a series, as defined in the field
\verb{Nombre del archivo con los valores de la serie} of the corresponding
catalog. See \code{\link[=bde_catalog_load]{bde_catalog_load()}}.}

\item{parse_dates}{Logical. If \code{TRUE}, dates are parsed using
\code{\link[=bde_parse_dates]{bde_parse_dates()}}.}

\item{parse_numeric}{Logical. If \code{TRUE} the columns would be parsed to
double (numeric) values. See \strong{Note}.}

\item{cache_dir}{A path to a cache directory. The directory can also be set
via options with \code{options(bde_cache_dir = "path/to/dir")}.}

\item{update_cache}{Logical. If \code{TRUE}, the requested file is updated in
\code{cache_dir}.}

\item{verbose}{Logical \code{TRUE} or \code{FALSE}, display information useful for
debugging.}

\item{extract_metadata}{Logical \code{TRUE/FALSE}. On \code{TRUE} the output is the
metadata of the requested series.}
}
\value{
A \link[tibble:tbl_df-class]{tibble} with a field \code{Date} and the alias of the fields
series as described on the catalogs. See \code{\link[=bde_catalog_load]{bde_catalog_load()}}.
}
\description{
Load a full time series file provided by BdE.
}
\details{
\subsection{About BdE file naming}{

The series name is a positional code showing the location of the table. For
example, table \strong{be_6_1} represents the Table 1, Chapter 6 of the
Statistical Bulletin ("BE"). Although it is a unique value, it is subject
to change (i.e. a new table is inserted before).

For that reason, the function \code{\link[=bde_series_load]{bde_series_load()}} is more suitable for
extracting specific time series.
}
}
\note{
This function tries to coerce the columns to numbers. For some series a
warning may be displayed if the parser fails. You can override the default
behavior with \code{parse_numeric = FALSE}
}
\examples{
\dontshow{if (bde_check_access()) withAutoprint(\{ # examplesIf}
\donttest{
# Metadata
bde_series_full_load("TI_1_1.csv", extract_metadata = TRUE)

# Data
bde_series_full_load("TI_1_1.csv")
}
\dontshow{\}) # examplesIf}
}
\seealso{
Other series: 
\code{\link{bde_series_load}()}
}
\concept{series}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\encoding{UTF-8}
\name{bde_ind_db}
\alias{bde_ind_db}
\title{Database of selected macroeconomic indicators}
\format{
A \link[tibble:tbl_df-class]{tibble} of 9 rows and
7 columns with the following fields:

\describe{
\item{tidyBdE_fun}{Function name, see \link{bde_indicators}.}
\item{Numero_secuencial}{Series code, see \code{\link[=bde_series_load]{bde_series_load()}}.}
\item{Descripcion_de_la_serie}{Description of the series in Spanish.}
\item{Fecha_de_la_primera_observacion}{Starting date of the indicator.}
\item{Fecha_de_la_ultima_observacion}{Most recent date available.}
\item{Fuente}{Data source.}
}
}
\description{
Minimal metadata of the selected macroeconomic indicators included in helper
functions of \CRANpkg{tidyBdE} (see \link{bde_indicators}). Full metadata can be
accessed via \code{\link[=bde_catalog_load]{bde_catalog_load()}}
}
\details{
\tabular{lllllll}{
   \strong{tidyBdE_fun} \tab \strong{Numero_secuencial} \tab \strong{Descripcion_de_la_serie} \tab \strong{Frecuencia_de_la_serie} \tab \strong{Fecha_de_la_primera_observacion} \tab \strong{Fecha_de_la_ultima_observacion} \tab \strong{Fuente} \cr
   bde_ind_cpi_var \tab 1489713 \tab Estadísticas Generales. IPCA. Base 2015. Índice general. Tasa interanual. España \tab MENSUAL \tab 1993-01-01 \tab 2025-12-01 \tab Eurostat \cr
   bde_ind_euribor_12m_daily \tab 905842 \tab Tipo de interés. UEM. Mercado monetario. Euríbor. A 12 meses \tab LABORABLE \tab 2000-01-03 \tab 2026-02-13 \tab REFINITIV \cr
   bde_ind_euribor_12m_monthly \tab 587853 \tab Tipo de interés. UEM. Mercado monetario. Euríbor. A 12 meses \tab MENSUAL \tab 1999-01-01 \tab 2026-01-01 \tab The European Money Market Institute (EMMI) \cr
   bde_ind_gdp_quarterly \tab 4663160 \tab Estadísticas Generales. Cuentas Nacionales. SEC2010. Año base 2020. Precios corrientes. Producto interior bruto. Economía en su conjunto (Total de la economía) (Saldo). Datos corregidos de efectos estacionales y de calendario. TRIMESTRAL \tab TRIMESTRAL \tab 1995-03-01 \tab 2025-12-01 \tab Instituto Nacional de Estadistica \cr
   bde_ind_gdp_var \tab 4663788 \tab Estadísticas Generales. CNTR. Base 2020. PIB. Precios constantes. Datos CVEC. Tasa interanual. España \tab TRIMESTRAL \tab 1996-03-01 \tab 2025-12-01 \tab Eurostat \cr
   bde_ind_ibex_daily \tab 821340 \tab Cotización y contratación. Acciones. Sociedad de Bolsas y Sociedad Rectora de la Bolsa de Madrid. Índice cotización. Indice IBEX 35 \tab LABORABLE \tab 1999-01-04 \tab 2026-02-13 \tab Bolsa de Madrid y Comisión Nacional del Mercado de Valores \cr
   bde_ind_ibex_monthly \tab 254433 \tab Cotización y contratación. Acciones. Sociedad de Bolsas y Sociedad Rectora de la Bolsa de Madrid. Índice cotización. Indice IBEX 35 \tab MENSUAL \tab 1987-01-01 \tab 2025-12-01 \tab SOCIEDAD RECTORA DE LA BOLSA DE MADRID \cr
   bde_ind_population \tab 4637737 \tab Estadísticas generales. INE. EPA. Base 2021. Total Nacional. Ambos sexos. Todas las edades. Personas. Trimestral \tab TRIMESTRAL \tab 2002-03-01 \tab 2025-09-01 \tab Instituto Nacional de Estadística \cr
   bde_ind_unemployment_rate \tab 4635980 \tab Estadísticas Generales. EPA. Base 2021. Total Nacional. Tasa de paro de la población. Ambos sexos. 16 y más años \tab TRIMESTRAL \tab 2002-03-01 \tab 2025-12-01 \tab Instituto Nacional de Estadística \cr
}
}
\examples{
data("bde_ind_db")
bde_ind_db

}
\seealso{
Other indicators: 
\code{\link{bde_indicators}}
}
\concept{indicators}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{bde_parse_dates}
\alias{bde_parse_dates}
\title{Parse dates}
\usage{
bde_parse_dates(dates_to_parse)
}
\arguments{
\item{dates_to_parse}{Dates to parse}
}
\value{
A \code{\link[=as.Date]{Date}} object.
}
\description{
This function is tailored for date formats used in this package and may fail
with other datasets. See \strong{Examples} for formats that are supported.
\subsection{Date Formats}{\tabular{lll}{
   \strong{FREQUENCY} \tab \strong{FORMAT} \tab \strong{EXAMPLES} \cr
   \strong{Daily / Business day} \tab DD MMMMYYYY \tab \emph{02 FEB2019} \cr
   \strong{Monthly} \tab MMM YYYY \tab \emph{MAR 2020} \cr
   \strong{Quarterly} \tab MMM YYYY, where MMM is the first or the last month of the quarter, depending on the value of its variable OBSERVED. \tab For the first quarter of 2020: \emph{ENE 2020, MAR 2020} \cr
   \strong{Half-yearly} \tab MMM YYYY, where MMM is the first or the last month of the halfyear period, depending on the value of its variable OBSERVED. \tab For the first half of 2020: \emph{ENE 2020, JUN 2020} \cr
   \strong{Annual} \tab YYYY \tab \emph{2020} \cr
}

}
}
\details{
Tries to parse strings representing dates using \code{\link[=as.Date]{as.Date()}}
}
\examples{
# Formats parsed
would_parse <- c(
  "02 FEB2019", "15 ABR 1890", "MAR 2020", "ENE2020",
  "2020", "12-1993", "01-02-2014", "01/02/1990"
)

parsed_ok <- bde_parse_dates(would_parse)

class(parsed_ok)

tibble::tibble(raw = would_parse, parsed = parsed_ok)

#-----------------------------------

# Formats not admitted
wont_parse <- c("JAN2001", "2010-01-12", "01 APR 2017", "01/31/1990")

parsed_fail <- bde_parse_dates(wont_parse)

class(parsed_fail)

tibble::tibble(raw = wont_parse, parsed = parsed_fail)
}
\seealso{
\code{\link[=as.Date]{as.Date()}}
}
\concept{utils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/catalogs.R
\encoding{UTF-8}
\name{bde_catalog_load}
\alias{bde_catalog_load}
\title{Load BdE catalogs}
\source{
\href{https://www.bde.es/webbe/en/estadisticas/recursos/descargas-completas.html}{time series bulk data download}.
}
\usage{
bde_catalog_load(
  catalog = c("ALL", "BE", "SI", "TC", "TI", "PB"),
  parse_dates = TRUE,
  cache_dir = NULL,
  update_cache = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{catalog}{A single value indicating the catalog to be updated or
\code{"ALL"} as a shorthand. See \strong{Details}.}

\item{parse_dates}{Logical. If \code{TRUE}, dates are parsed using
\code{\link[=bde_parse_dates]{bde_parse_dates()}}.}

\item{cache_dir}{A path to a cache directory. The directory can also be set
via options with \code{options(bde_cache_dir = "path/to/dir")}.}

\item{update_cache}{Logical. If \code{TRUE}, the requested file is updated in
\code{cache_dir}.}

\item{verbose}{Logical \code{TRUE} or \code{FALSE}, display information useful for
debugging.}
}
\value{
A \link[tibble:tbl_df-class]{tibble} object.
}
\description{
Load the time series catalogs provided by the BdE.
}
\details{
Accepted values for \code{catalog} are:\tabular{llll}{
   \strong{CODE} \tab \strong{PUBLICATION} \tab \strong{UPDATE FREQUENCY} \tab \strong{FREQUENCY} \cr
   \code{"BE"} \tab Statistical Bulletin \tab Daily \tab Monthly \cr
   \code{"SI"} \tab Summary Indicators \tab Daily \tab Daily \cr
   \code{"TC"} \tab Exchange Rates \tab Daily \tab Daily \cr
   \code{"TI"} \tab Interest Rates \tab Daily \tab Daily \cr
   \code{"PB"} \tab Bank Lending Survey \tab Quarterly \tab Quarterly \cr
}


Use \code{"ALL"} as a shorthand for updating all the catalogs at a glance.

If the requested catalog is not cached, \code{\link[=bde_catalog_update]{bde_catalog_update()}} is invoked.
}
\examples{
\dontshow{if (bde_check_access()) withAutoprint(\{ # examplesIf}
\donttest{
bde_catalog_load("TI", verbose = TRUE)
}
\dontshow{\}) # examplesIf}
}
\seealso{
Other catalog: 
\code{\link{bde_catalog_search}()},
\code{\link{bde_catalog_update}()}
}
\concept{catalog}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bde_tidy_palettes.R
\name{bde_tidy_palettes}
\alias{bde_tidy_palettes}
\title{BdE color palettes}
\usage{
bde_tidy_palettes(
  n = 6,
  palette = c("bde_vivid_pal", "bde_rose_pal", "bde_qual_pal"),
  alpha = NULL,
  rev = FALSE
)
}
\arguments{
\item{n}{The number of colors (\verb{>= 1}) in the palette.}

\item{palette}{A valid palette name.}

\item{alpha}{An alpha-transparency level in the range \verb{[0,1]} (\code{0} means
transparent and \code{1} means opaque). A missing, i.e., \code{alpha = NULL}, does
not add opacity codes (\code{"FF"}) to the individual color hex codes.
See \code{\link[ggplot2:reexports]{ggplot2::alpha()}}.}

\item{rev}{Logical indicating whether the ordering of the colors should be
reversed.}
}
\value{
A vector of colors.
}
\description{
Custom palettes based on the publications of BdE. These are manual
palettes with a maximum of 6 colors.
}
\examples{

# BdE vivid pal
scales::show_col(bde_tidy_palettes(palette = "bde_vivid_pal"),
  labels = FALSE
)

# BdE rose pal
scales::show_col(bde_tidy_palettes(palette = "bde_rose_pal"),
  labels = FALSE
)

# BdE qual pal
scales::show_col(bde_tidy_palettes(palette = "bde_qual_pal"),
  labels = FALSE
)
}
\seealso{
Other bde_plot: 
\code{\link{scales_bde}},
\code{\link{theme_tidybde}()}
}
\concept{bde_plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/series.R
\encoding{UTF-8}
\name{bde_series_load}
\alias{bde_series_load}
\title{Load a single BdE time series}
\usage{
bde_series_load(
  series_code,
  series_label = NULL,
  out_format = "wide",
  parse_dates = TRUE,
  parse_numeric = TRUE,
  cache_dir = NULL,
  update_cache = FALSE,
  verbose = FALSE,
  extract_metadata = FALSE
)
}
\arguments{
\item{series_code}{A numeric value (or coercible with \code{\link[base:double]{base::as.double()}})
or vector of time series code(s), as defined in the field
\verb{Número secuencial} of the corresponding series. See \code{\link[=bde_catalog_load]{bde_catalog_load()}}.}

\item{series_label}{Optional. Character vector or value. Allows specifying a
custom label for the series extracted.}

\item{out_format}{Whether the format should be returned as "long" or "wide".
Possible values are \code{"wide"} or \code{"long"}. See \strong{Value} for details and
section \strong{Examples}.}

\item{parse_dates}{Logical. If \code{TRUE}, dates are parsed using
\code{\link[=bde_parse_dates]{bde_parse_dates()}}.}

\item{parse_numeric}{Logical. If \code{TRUE} the columns would be parsed to
double (numeric) values. See \strong{Note}.}

\item{cache_dir}{A path to a cache directory. The directory can also be set
via options with \code{options(bde_cache_dir = "path/to/dir")}.}

\item{update_cache}{Logical. If \code{TRUE}, the requested file is updated in
\code{cache_dir}.}

\item{verbose}{Logical \code{TRUE} or \code{FALSE}, display information useful for
debugging.}

\item{extract_metadata}{Logical \code{TRUE/FALSE}. On \code{TRUE} the output is the
metadata of the requested series.}
}
\value{
A \link[tibble:tbl_df-class]{tibble} with a field \code{Date}:
\itemize{
\item With \code{out_format = "wide"}, each series is presented in a separate
column with the name defined by \code{series_label}.
\item With \code{out_format = "long"}, the tibble has two additional columns:
\itemize{
\item \code{serie_name} with the label of each series.
\item \code{serie_value} with the corresponding value.
}
}

\code{"wide"} format is more suitable for exporting to a \code{.csv} file while
\code{"long"} format is more suitable for creating plots with
\code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}. See also \code{\link[tidyr:pivot_longer]{tidyr::pivot_longer()}} and
\code{\link[tidyr:pivot_wider]{tidyr::pivot_wider()}}.
}
\description{
The series alias is a positional code showing the location (column and/or
row) of the series in the table. However, although it is unique, it is not a
good candidate to be used as the series ID, as it is subject to change. If a
series changes position in the table, its alias will also change.

To ensure series can still be identified, even after these changes, they are
assigned a sequential number (referred to as \code{series_code} in this function).

Note that a single series may appear in different tables, so it can have
several aliases. If you need to search by alias, use
\code{\link[=bde_series_full_load]{bde_series_full_load()}}.
}
\details{
Load a single time series provided by BdE.
}
\note{
This function attempts to coerce the columns to numbers. For some series, a
warning may be displayed if the parsing fails.
}
\examples{
\dontshow{if (bde_check_access()) withAutoprint(\{ # examplesIf}
\donttest{
# Metadata
bde_series_load(573234, verbose = TRUE, extract_metadata = TRUE)

# Data
bde_series_load(573234, extract_metadata = FALSE)

# Vectorized
bde_series_load(c(573234, 573214),
  series_label = c("US/EUR", "GBP/EUR"),
  extract_metadata = TRUE
)

wide <- bde_series_load(c(573234, 573214),
  series_label = c("US/EUR", "GBP/EUR")
)

# Wide format
wide

# Long format
long <- bde_series_load(c(573234, 573214),
  series_label = c("US/EUR", "GBP/EUR"),
  out_format = "long"
)

long

# Use with ggplot
library(ggplot2)

ggplot(long, aes(Date, serie_value)) +
  geom_line(aes(group = serie_name, color = serie_name)) +
  scale_color_bde_d() +
  theme_tidybde()
}
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link[=bde_catalog_load]{bde_catalog_load()}},
\code{\link[=bde_catalog_search]{bde_catalog_search()}}, \code{\link[=bde_indicators]{bde_indicators()}}

Other series: 
\code{\link{bde_series_full_load}()}
}
\concept{series}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indicators.R
\encoding{UTF-8}
\name{bde_indicators}
\alias{bde_indicators}
\alias{bde_ind_gdp_var}
\alias{bde_ind_unemployment_rate}
\alias{bde_ind_euribor_12m_monthly}
\alias{bde_ind_euribor_12m_daily}
\alias{bde_ind_cpi_var}
\alias{bde_ind_ibex_monthly}
\alias{bde_ind_ibex_daily}
\alias{bde_ind_gdp_quarterly}
\alias{bde_ind_population}
\alias{bde_ind_ibex}
\title{Relevant Indicators of Spain}
\usage{
bde_ind_gdp_var(series_label = "GDP_YoY", ...)

bde_ind_unemployment_rate(series_label = "Unemployment_Rate", ...)

bde_ind_euribor_12m_monthly(series_label = "Euribor_12M_Monthly", ...)

bde_ind_euribor_12m_daily(series_label = "Euribor_12M_Daily", ...)

bde_ind_cpi_var(series_label = "Consumer_price_index_YoY", ...)

bde_ind_ibex_monthly(series_label = "IBEX_index_month", ...)

bde_ind_ibex_daily(series_label = "IBEX_index_day", ...)

bde_ind_gdp_quarterly(series_label = "GDP_quarterly_value", ...)

bde_ind_population(series_label = "Population_Spain", ...)
}
\arguments{
\item{series_label}{Optional. Character vector or value. Allows specifying a
custom label for the series extracted.}

\item{...}{
  Arguments passed on to \code{\link[=bde_series_load]{bde_series_load}}
  \describe{
    \item{\code{out_format}}{Whether the format should be returned as "long" or "wide".
Possible values are \code{"wide"} or \code{"long"}. See \strong{Value} for details and
section \strong{Examples}.}
    \item{\code{parse_numeric}}{Logical. If \code{TRUE} the columns would be parsed to
double (numeric) values. See \strong{Note}.}
    \item{\code{extract_metadata}}{Logical \code{TRUE/FALSE}. On \code{TRUE} the output is the
metadata of the requested series.}
    \item{\code{parse_dates}}{Logical. If \code{TRUE}, dates are parsed using
\code{\link[=bde_parse_dates]{bde_parse_dates()}}.}
    \item{\code{update_cache}}{Logical. If \code{TRUE}, the requested file is updated in
\code{cache_dir}.}
    \item{\code{cache_dir}}{A path to a cache directory. The directory can also be set
via options with \code{options(bde_cache_dir = "path/to/dir")}.}
    \item{\code{verbose}}{Logical \code{TRUE} or \code{FALSE}, display information useful for
debugging.}
  }}
}
\value{
A \link[tibble:tbl_df-class]{tibble} with the required series.
}
\description{
Set of helper functions for downloading some of the most relevant
macroeconomic indicators of Spain. Metadata available in \link{bde_ind_db}.
}
\details{
These functions are convenient wrappers around \code{\link[=bde_series_load]{bde_series_load()}} for
specific series. Use \verb{verbose = TRUE, extract_metadata = TRUE} to see the
specification and the source.
}
\examples{
\dontshow{if (bde_check_access()) withAutoprint(\{ # examplesIf}
\donttest{
bde_ind_gdp_var()
}
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link[=bde_series_load]{bde_series_load()}}, \code{\link[=bde_catalog_search]{bde_catalog_search()}}

Other indicators: 
\code{\link{bde_ind_db}}
}
\concept{indicators}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/catalogs.R
\encoding{UTF-8}
\name{bde_catalog_search}
\alias{bde_catalog_search}
\title{Search BdE catalogs}
\usage{
bde_catalog_search(pattern, ...)
}
\arguments{
\item{pattern}{\code{\link[base:regex]{regex}} pattern to search. See \strong{Details}
and \strong{Examples}.}

\item{...}{
  Arguments passed on to \code{\link[=bde_catalog_load]{bde_catalog_load}}
  \describe{
    \item{\code{catalog}}{A single value indicating the catalog to be updated or
\code{"ALL"} as a shorthand. See \strong{Details}.}
    \item{\code{parse_dates}}{Logical. If \code{TRUE}, dates are parsed using
\code{\link[=bde_parse_dates]{bde_parse_dates()}}.}
    \item{\code{update_cache}}{Logical. If \code{TRUE}, the requested file is updated in
\code{cache_dir}.}
    \item{\code{cache_dir}}{A path to a cache directory. The directory can also be set
via options with \code{options(bde_cache_dir = "path/to/dir")}.}
    \item{\code{verbose}}{Logical \code{TRUE} or \code{FALSE}, display information useful for
debugging.}
  }}
}
\value{
A \link[tibble:tbl_df-class]{tibble} object with the results of the query.
}
\description{
Search for keywords on the time series catalogs.
}
\details{
\strong{Note:} BdE files are currently provided only in Spanish. Therefore, search
terms should be provided in Spanish to obtain search results.

This function uses \code{\link[base:grep]{base::grep()}} function for finding matches on the
catalogs. You can pass \link[base:regex]{regular expressions} to broaden the
search.
}
\examples{
\dontshow{if (bde_check_access()) withAutoprint(\{ # examplesIf}
\donttest{
# Simple search (needs to be in Spanish)
# !! PIB [es] == GDP [en]

bde_catalog_search("PIB")

# More complex - Single
bde_catalog_search("Francia(.*)PIB")

# Even more complex - Double
bde_catalog_search("Francia(.*)PIB|Italia(.*)PIB|Alemania(.*)PIB")
}
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link[=bde_catalog_load]{bde_catalog_load()}}, \link[base:regex]{base::regex}

Other catalog: 
\code{\link{bde_catalog_load}()},
\code{\link{bde_catalog_update}()}
}
\concept{catalog}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_tidybde.R
\name{theme_tidybde}
\alias{theme_tidybde}
\title{BdE \CRANpkg{ggplot2} theme}
\usage{
theme_tidybde(...)
}
\arguments{
\item{...}{
  Arguments passed on to \code{\link[ggplot2:ggtheme]{ggplot2::theme_classic}}
  \describe{
    \item{\code{base_size}}{base font size, given in pts.}
    \item{\code{base_family}}{base font family}
    \item{\code{header_family}}{font family for titles and headers. The default, \code{NULL},
uses theme inheritance to set the font. This setting affects axis titles,
legend titles, the plot title and tag text.}
    \item{\code{base_line_size}}{base size for line elements}
    \item{\code{base_rect_size}}{base size for rect elements}
    \item{\code{ink,paper,accent}}{colour for foreground, background, and accented elements respectively.}
  }}
}
\value{
A \CRANpkg{ggplot2}
\code{\link[ggplot2:ggtheme]{theme()}}.
}
\description{
A custom \CRANpkg{ggplot2} theme based on the publications
of BdE.
}
\details{
Theme based on \code{\link[ggplot2:ggtheme]{ggplot2::theme_classic()}}.
}
\examples{
\dontshow{if (bde_check_access()) withAutoprint(\{ # examplesIf}
\donttest{
library(ggplot2)
library(dplyr)
library(tidyr)

series_TC <- bde_series_full_load("TC_1_1.csv")

# If download was OK then plot
if (nrow(series_TC) > 0) {
  series_TC <- series_TC[c(1, 2)]

  series_TC_pivot <- series_TC |>
    filter(
      Date >= "2020-01-01" & Date <= "2020-12-31",
      !is.na(series_TC[[2]])
    )

  names(series_TC_pivot) <- c("x", "y")

  ggplot(series_TC_pivot, aes(x = x, y = y)) +
    geom_line(linewidth = 0.8, color = bde_tidy_palettes(n = 1)) +
    labs(
      title = "Title",
      subtitle = "Some metric",
      caption = "Bank of Spain"
    ) +
    theme_tidybde()
}
}
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link[ggplot2:ggtheme]{ggplot2::theme_classic()}}

Other bde_plot: 
\code{\link{bde_tidy_palettes}()},
\code{\link{scales_bde}}
}
\concept{bde_plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidyBdE-package.R
\docType{package}
\name{tidyBdE-package}
\alias{tidyBdE}
\alias{tidyBdE-package}
\title{tidyBdE: Download Data from Bank of Spain}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Tools to download data series from 'Banco de España' ('BdE') on 'tibble' format. 'Banco de España' is the national central bank and, within the framework of the Single Supervisory Mechanism ('SSM'), the supervisor of the Spanish banking system along with the European Central Bank. This package is in no way sponsored endorsed or administered by 'Banco de España'.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ropenspain.github.io/tidyBdE/}
  \item \url{https://github.com/rOpenSpain/tidyBdE}
  \item Report bugs at \url{https://github.com/rOpenSpain/tidyBdE/issues}
}

}
\author{
\strong{Maintainer}: Diego H. Herrero \email{dev.dieghernan@gmail.com} (\href{https://orcid.org/0000-0001-8457-4658}{ORCID}) [copyright holder]

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bde_check_access.R
\name{bde_check_access}
\alias{bde_check_access}
\title{Check access to BdE}
\usage{
bde_check_access()
}
\value{
A logical value.
}
\description{
Check whether \strong{R} can access resources at
\url{https://www.bde.es/webbe/en/estadisticas/recursos/descargas-completas.html}.
}
\examples{
\donttest{
bde_check_access()
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/superseded.R
\name{bde_vivid_pal}
\alias{bde_vivid_pal}
\alias{bde_rose_pal}
\title{BdE superseded palettes}
\usage{
bde_vivid_pal(...)

bde_rose_pal(...)
}
\arguments{
\item{...}{Further arguments of the functions.}
}
\value{
A palette of colors.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#superseded}{\figure{lifecycle-superseded.svg}{options: alt='[Superseded]'}}}{\strong{[Superseded]}}

These functions have been superseded; see \code{\link[=bde_tidy_palettes]{bde_tidy_palettes()}} as a
replacement.

Custom palettes based on the publications of BdE.
}
\examples{

# BdE vivid pal
scales::show_col(bde_vivid_pal()(6), labels = FALSE)

# BdE rose pal
scales::show_col(bde_rose_pal()(6), labels = FALSE)
}
\keyword{internal}
